// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package alarm

import (
	"context"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"smart-site-management/utils"

	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
)

func AlarmContactAdd(ctx context.Context, req *v1.AlarmContactAddRequest) (reply *v1.AlarmContactAddReply, err error) {
	if req.ProjectId == 0 {
		return nil, errors.ParamsError
	}
	p := model.TAlarmContact{}
	where := map[string]interface{}{
		"ProjectId": req.ProjectId,
	}
	or := map[string]interface{}{}
	if req.Phone != "" {
		or["Phone"] = req.Phone
	}
	if req.Email != "" {
		or["Email"] = req.Email
	}
	err = p.Find(database.DB(), where, or)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("AlarmContactAdd",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	if p.Id > 0 {
		return nil, errors.AlarmContactExistError
	}

	err = utils.StructCopy(&p, req, "")
	if err != nil {
		logger.Error("AlarmContactAdd",
			zap.String("err", err.Error()))
		return nil, errors.SystemError
	}

	err = p.Insert(database.DB())
	if err != nil {
		logger.Error("AlarmContactAdd",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	// TODO 设置告警联系人到Redis
	//utils.SetAlarmContact(&p)

	reply = &v1.AlarmContactAddReply{Id: int64(p.Id)}
	return reply, nil

}
