// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package alarm

import (
	"context"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"smart-site-management/utils"

	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"github.com/jaryhe/gopkgs/logger"
)

func AlarmContactList(ctx context.Context, req *v1.AlarmContactListRequest) (reply *v1.AlarmContactListReply, err error) {

	if req.ProjectId == 0 {
		return nil, errors.ParamsError
	}
	p := model.TAlarmContact{}
	where := map[string]interface{}{
		"ProjectId": req.ProjectId,
	}

	reply = &v1.AlarmContactListReply{}
	list, err := p.QueryAll(database.DB(), where)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return &v1.AlarmContactListReply{}, nil
		}
		logger.Error("AlarmContactList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	reply = &v1.AlarmContactListReply{}
	reply.List = make([]*v1.AlarmContact, len(list))
	for i := 0; i < len(list); i++ {
		item := v1.AlarmContact{}
		err := utils.StructCopy(&item, &list[i], "")
		if err != nil {
			return nil, errors.SystemError
		}
		item.Id = int64(list[i].Id)
		reply.List[i] = &item
	}

	return reply, nil
}
