// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package alarm

import (
	"context"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"

	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"github.com/jaryhe/gopkgs/logger"
)

func AlarmContactUpdate(ctx context.Context, req *v1.AlarmContactUpdateRequest) (reply *v1.AlarmContactUpdateReply, err error) {
	if req.Contact.Id == 0 {
		return nil, errors.ParamsError
	}
	p := model.TAlarmContact{}
	where := map[string]interface{}{
		"ID": req.Contact.Id,
		//"project_id":req.Contact.ProjectId,
	}
	err = p.Query(database.DB(), where)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return nil, errors.AlarmContactNotExistError
		}
		logger.Error("AlarmContactUpdate",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	if p.ProjectId != req.Contact.ProjectId {
		return nil, errors.ProjectNotMatchError
	}

	/*p.Id = req.Contact.Id
	err = utils.StructCopy(&p, req.Rule, "")
	if err != nil {
		return nil, errors.SystemError
	}*/

	fileds := map[string]interface{}{
		"Phone": req.Contact.Phone,
		"Email": req.Contact.Email,
	}

	err = p.UpdateSome(database.DB(), fileds)
	if err != nil {
		logger.Error("AlarmContactUpdate",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	// 设置告警规则到内存
	//utils.SetAlarmRule(&p)

	reply = &v1.AlarmContactUpdateReply{}
	return reply, nil
}
