// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package alarm

import (
	"context"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"smart-site-management/utils"

	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"github.com/jaryhe/gopkgs/logger"
)

func AlarmRuleAdd(ctx context.Context, req *v1.AlarmRuleAddRequest) (reply *v1.AlarmRuleAddReply, err error) {
	if req.Sn == "" || req.ProjectId == 0 {
		return nil, errors.ParamsError
	}
	p := model.TAlarmRule{}
	where := map[string]interface{}{
		"ProjectId": req.ProjectId,
		"SN":         req.Sn,
	}
	err = p.Query(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("AlarmRuleAdd",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.Id > 0 {
		return nil, errors.AlarmRuleExistError
	}

	err = utils.StructCopy(&p, req, "")
	if err != nil {
		logger.Error("AlarmRuleAdd",
			zap.String("err", err.Error()))
		return nil, errors.SystemError
	}

	err = p.Insert(database.DB())
	if err != nil {
		logger.Error("AlarmRuleAdd",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	// 设置告警规则到内存Redis
	//utils.SetAlarmRule(&p)

	reply = &v1.AlarmRuleAddReply{Id: int64(p.Id)}
	return reply, nil
}
