// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package alarm

import (
	"context"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"smart-site-management/utils"

	"github.com/jaryhe/gopkgs/database"
	"go.uber.org/zap"
	"github.com/jaryhe/gopkgs/logger"
)

func AlarmRuleList(ctx context.Context, req *v1.AlarmRuleListRequest) (reply *v1.AlarmRuleListReply, err error) {

	if req.ProjectId == 0 {
		return nil, errors.ParamsError
	}
	if req.Page < 1 {
		req.Page = 1
	}
	p := model.TAlarmRule{}
	list, total, err := p.List(database.DB(), req.Sn, req.ProjectId, req.Page)
	if err != nil {
		logger.Error("AlarmRuleList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply = &v1.AlarmRuleListReply{
		Page:req.Page,
		PageSize:int32(model.PageSize),
		Total:total,
	}
	if total == 0 {
		return reply, nil
	}
	reply.List = make([]*v1.AlarmRule, len(list))
	for i := 0; i < len(list); i++ {
		item := v1.AlarmRule{}
		err := utils.StructCopy(&item, &list[i].TAlarmRule, "")
		if err != nil {
			logger.Error("AlarmRuleList",
				zap.String("err", err.Error()))
			return nil, errors.SystemError
		}
		item.Id = list[i].Id
		//item.IsOn = list[i].IsOn
		item.DeviceName = list[i].DeviceName
		item.ProjectName = list[i].ProjectName
		reply.List[i] = &item
	}

	return reply, nil
}
