// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package alarm

import (
	"context"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"

	"github.com/jinzhu/gorm"

	"github.com/jaryhe/gopkgs/database"
	"go.uber.org/zap"
	"github.com/jaryhe/gopkgs/logger"
)

func AlarmRuleOnOff(ctx context.Context, req *v1.AlarmRuleOnOffRequest) (reply *v1.AlarmRuleOnOffReply, err error) {
	if req.Id == 0 {
		return nil, errors.ParamsError
	}

	// 判断告警规则是否存在
	p := model.TAlarmRule{}
	where := map[string]interface{}{
		"ID": req.Id,
	}
	err = p.Query(database.DB(), where)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return nil, errors.AlarmRuleNotExistError
		}
		logger.Error("AlarmRuleOnOff",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	// 更新启停状态
	values := map[string]interface{}{
		"IsOn": req.IsOn,
	}
	err = p.UpdateSome(database.DB(), values)
	if err != nil {
		logger.Error("AlarmRuleOnOff",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	/*
		if req.IsOn == true {
			// 设置告警规则到内存
			p.IsOn = true
			utils.SetAlarmRule(&p)
		} else {
			utils.DeleteAlarmRule(p.Sn)
		}
	*/
	reply = &v1.AlarmRuleOnOffReply{}
	return reply, nil

}
