// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package camera

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"smart-site-management/consts"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"strings"
)

func ChannelListByProject(ctx context.Context, req *v1.ChannelListRequest) (reply *v1.ChannelListReply, err error) {
	list, total, err := model.ChannelListByProject(database.DB(), req.ProjectId, req.Page, req.PageSise, req.ChannelNo, req.IsUnbind)
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply = &v1.ChannelListReply{}
	reply.PageSize = req.PageSise
	reply.Page = req.Page
	reply.Total = total
	if total == 0 {
		return reply, nil
	}
	reply.List = make([]*v1.ChannelItem, len(list))
	for i, v := range list {
		item := &v1.ChannelItem{
			Name:v.Nickname,
			Id:v.ID,
			ChannelNo:v.Chanpubid,
			State:int32(v.Alive),
		}
		reply.List[i] = item
	}
	if int64(req.Page - 1) * int64(req.PageSise) + int64(len(reply.List)) < reply.Total {
		reply.HasMore = true
	}

	return reply, nil

}

func ChannelList(ctx context.Context, req *v1.ChannelListRequest) (reply *v1.ChannelListReply, err error) {
	if req.Page < 1 {
		req.Page = 1
	}
	if req.PageSise == 0 {
		req.PageSise = 10
	}
	if req.Sn == "" && req.ProjectId == 0 {
		return nil, errors.ParamsError
	}
	if req.ChannelNo != "" {
		req.ChannelNo = strings.ToLower(req.ChannelNo)
	}
	if req.Sn == "" {
		return ChannelListByProject(ctx, req)
	}

	p := model.DeviceAll{}
	where := map[string]interface{}{
		"SN":req.Sn,
	}
	err = p.Find(database.DB(), where)
	if err != nil {
		logger.Error("ChannelList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.NoRecordError
	}
	if p.VerifyStatus != consts.DeviceStatusAddAuditted {
		return nil, errors.VssStatusError
	}
	pc := model.Vsschanneltbl{}
	where = map[string]interface{}{
		"DevPubID":req.Sn,
	}

	or := map[string]interface{}{}
	if req.ChannelNo != "" {
		or["ChanPubID like"] = "%"+req.ChannelNo+"%"
		or["Nickname like"] = "%"+req.ChannelNo+"%"
	}
	count, err := pc.Count(database.DB(), where, or)
	if err != nil {
		logger.Error("ChannelList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply = &v1.ChannelListReply{
		Page:req.Page,
		PageSize:int32(model.PageSize),
		Total:count,
	}
	if count == 0 {
		return reply, nil
	}

	list, err := pc.List(database.DB(), where, or, req.Page, req.PageSise)
	if err != nil {
		logger.Error("ChannelList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply.List = make([]*v1.ChannelItem, len(list))
	for i, v := range list {
		item := &v1.ChannelItem{
			Name:v.Nickname,
			Id:v.ID,
			ChannelNo:v.Chanpubid,
			State:int32(v.Alive),
		}
		reply.List[i] = item
	}
	if int64(req.Page - 1) * int64(req.PageSise) + int64(len(reply.List)) < reply.Total {
		reply.HasMore = true
	}

	return reply, nil

}
