// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package camera

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
)


func dustBindCamera(db *gorm.DB, Id int64, ChannelId int64) error {
	p := model.DustDeviceInfo{}
	where := map[string]interface{}{
		"ID":Id,
	}
	values := map[string]interface{}{
		"ChannelId":ChannelId,
	}
	return p.Update(db, where, values)
}

func slagCarBindCamera(db *gorm.DB, Id int64, ChannelId int64) error {
	p := model.SlagcarDevice{}
	where := map[string]interface{}{
		"ID":Id,
	}
	values := map[string]interface{}{
		"ChannelId":ChannelId,
	}
	return p.Update(db, where, values)
}

func towerBindCamera(db *gorm.DB, Id int64, ChannelId int64) error {
	p := model.TowerDeviceInfo{}
	where := map[string]interface{}{
		"ID":Id,
	}
	values := map[string]interface{}{
		"ChannelId":ChannelId,
	}
	return p.Update(db, where, values)
}

func DeviceBindCamera(ctx context.Context, req *v1.DeviceBindCameraRequest) (reply *v1.DeviceBindCameraReply, err error) {
	reply  = &v1.DeviceBindCameraReply{}
	if req.DeviceId < 1 || req.ChannelId < 1 || req.DeviceCode == 0 {
		return nil, errors.ParamsError
	}
	p := model.DeviceAll{}
	// 判断是否已绑定摄像头
	where := map[string]interface{}{
		"DeviceCode !=":model.DeviceTypeVedio,
		"ChannelId":req.ChannelId,
	}
	count, err := p.Count(database.DB(), where)
	if err != nil {
		logger.Error("DeviceBindCamera",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if count > 0 {
		return nil, errors.DeviceExistError
	}
	pc := model.Vsschanneltbl{}
	// 判断摄像头是否存在
	where = map[string]interface{}{
		"ID":req.ChannelId,
	}
	count, err = pc.Count(database.DB(), where, nil)
	if err != nil {
		logger.Error("DeviceBindCamera",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if count == 0 {
		return nil, errors.NoRecordError
	}

	// 判断设备是否存在或可用
	where = map[string]interface{}{
		"SubId":req.DeviceId,
		"VerifyStatus":1,
		"DeviceCode":req.DeviceCode,
	}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound  {
		logger.Error("DeviceBindCamera",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil,  status.Error(10005, "设备未通过审核")
	}
	if p.DeviceCode == model.DeviceTypeVedio {
		return nil, errors.CameraBindError
	}

	values := map[string]interface{}{
		"ChannelId":req.ChannelId,
	}
	// 更新总表
	db := database.DB().Begin()
	err = p.Update(db, where, values)
	if err != nil {
		logger.Error("DeviceBindCamera",
			zap.String("err", err.Error()))
		db.Rollback()
		return nil, errors.DataBaseError
	}

	// 更新子表
	switch p.DeviceCode {
	case model.DeviceTypeDust:
		err = dustBindCamera(db, req.DeviceId, req.ChannelId)
	case model.DeviceTypeSlag:
		err = slagCarBindCamera(db, req.DeviceId, req.ChannelId)
	case model.DeviceTypeTower:
		err = towerBindCamera(db, req.DeviceId, req.ChannelId)
	}
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	db.Commit()
	return reply, nil
}
