// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package camera

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
)

func dustUnBindCamera(db *gorm.DB, Id int64) error {
	p := model.DustDeviceInfo{}
	where := map[string]interface{}{
		"ID":Id,
	}
	values := map[string]interface{}{
		"ChannelId":0,
	}
	return p.Update(db, where, values)
}

func slagCarUnBindCamera(db *gorm.DB, Id int64) error {
	p := model.SlagcarDevice{}
	where := map[string]interface{}{
		"ID":Id,
	}
	values := map[string]interface{}{
		"ChannelId":0,
	}
	return p.Update(db, where, values)
}

func towerUnBindCamera(db *gorm.DB, Id int64) error {
	p := model.TowerDeviceInfo{}
	where := map[string]interface{}{
		"ID":Id,
	}
	values := map[string]interface{}{
		"ChannelId":0,
	}
	return p.Update(db, where, values)
}


func DeviceUnbindCamera(ctx context.Context, req *v1.DeviceUnbindCameraRequest) (reply *v1.DeviceUnbindCameraReply, err error) {
	reply  = &v1.DeviceUnbindCameraReply{}
	if req.DeviceId < 1 || req.DeviceCode == 0 {
		return nil, errors.ParamsError
	}
	p := model.DeviceAll{}
	where := map[string]interface{}{
		"SubId":req.DeviceId,
		"DeviceCode":req.DeviceCode,
	}

	err = p.Find(database.DB(), where)
	if err != nil  {
		logger.Error("DeviceUnbindCamera",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.NoRecordError
	}

	values := map[string]interface{}{
		"ChannelId":0,
	}
	db := database.DB().Begin()
	err = p.Update(db, where, values)
	if err != nil {
		logger.Error("DeviceUnbindCamera",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}


	// 更新子表
	switch p.DeviceCode {
	case model.DeviceTypeDust:
		err = dustUnBindCamera(db, req.DeviceId)
	case model.DeviceTypeSlag:
		err = slagCarUnBindCamera(db, req.DeviceId)
	case model.DeviceTypeTower:
		err = towerUnBindCamera(db, req.DeviceId)
	}
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	db.Commit()

	return reply, nil
}
