// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package camera

import (
	"context"
	"fmt"
	//"github.com/tidwall/gjson"
	"io/ioutil"
	"net/http"
	"smart-site-management/errors"
	"smart-site-management/pb/v1"
	"time"
	"smart-site-management/parser"
	//"google.golang.org/grpc/status"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
)

func VedioPing(ctx context.Context, req *v1.VedioPingRequest) (reply *v1.VedioPingReply, err error) {
	reply = &v1.VedioPingReply{}
	if req.Sessionid == "" {
		return nil, errors.ParamsError
	}

	if parser.Conf.VedioPingUrl == "" {
		return nil, errors.ConfError
	}

	client := &http.Client{}
	timeOut := 30

	client.Timeout = time.Duration(timeOut) * time.Second

	hreq, err := http.NewRequest("GET", fmt.Sprintf("%s?sessionid=%s", parser.Conf.VedioPingUrl, req.Sessionid), nil)
	if err != nil {
		logger.Error("VedioPing",
			zap.String("err", err.Error()))
		return nil, err
	}

	resp, err := client.Do(hreq)
	if err != nil {
		logger.Error("VedioPing",
			zap.String("err", err.Error()))
		return nil, errors.VendorError
	}
	defer resp.Body.Close()
	result, err := ioutil.ReadAll(resp.Body)
	if len(result) > 0 {
		reply.Result = string(result)
		return reply, nil
	}
	return reply, nil
	/*

	code := gjson.GetBytes(result, "errcode").Int()
	errmsg := gjson.GetBytes(result, "errdesc").String()
	if code != 0 {
		logger.Error("VedioPing",
			zap.String("err", errmsg))
		return nil, status.Error(20018, "三方错误:"+errmsg)
	}
	return reply, nil

	 */
}
