// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package camera

import (
	"context"
	"fmt"
	"github.com/tidwall/gjson"
	"io/ioutil"
	"net/http"
	"smart-site-management/errors"
	"smart-site-management/pb/v1"
	"time"
	"smart-site-management/parser"
	"google.golang.org/grpc/status"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
)

var ptzMap  = map[string]bool{
	"up":true,
	"down":true,
	"left":true,
	"right":true,
	"leftup":true,
	"leftdown":true,
	"rightup":true,
	"rightdown":true,
	"zoomin":true,
	"zoomout":true,
	"focusfar":true,
	"focusnear":true,
	"stop":true,
	"callpos":true,
	"setpos":true,
	"delpos":true,
}

func Ptz(ctx context.Context, req *v1.PtzRequest) (reply *v1.PtzReply, err error) {
	reply = &v1.PtzReply{}
	if req.ChannelNo == "" || req.Ptzparam < 1{
		return nil, errors.ParamsError
	}
	if _, ok := ptzMap[req.Ptztype]; !ok {
		return nil, errors.ParamsError
	}
	if parser.Conf.PtzUrl == "" {
		return nil, errors.ConfError
	}

	client := &http.Client{}
	timeOut := 30

	client.Timeout = time.Duration(timeOut) * time.Second

	hreq, err := http.NewRequest("GET", fmt.Sprintf("%s?chanpubid=%s&ptztype=%s&ptzparam=%d", parser.Conf.PtzUrl, req.ChannelNo, req.Ptztype, req.Ptzparam), nil)
	if err != nil {
		logger.Error("Ptz",
			zap.String("err", err.Error()))
		return nil, err
	}

	resp, err := client.Do(hreq)
	if err != nil {
		logger.Error("Ptz",
			zap.String("err", err.Error()))
		return nil, errors.VendorError
	}
	defer resp.Body.Close()

	result, err := ioutil.ReadAll(resp.Body)
	code := gjson.GetBytes(result, "errcode").Int()
	errmsg := gjson.GetBytes(result, "errdesc").String()
	if code != 0 {
		logger.Error("Ptz",
			zap.String("err", errmsg))
		return nil, status.Error(20018, "三方错误:"+errmsg)
	}
	return reply, nil
}
