// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package camera

import (
	"context"
	"fmt"
	"github.com/tidwall/gjson"
	"io/ioutil"
	"net/http"
	"smart-site-management/errors"
	"smart-site-management/pb/v1"
	"time"
	"smart-site-management/parser"
	"google.golang.org/grpc/status"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
)

func RealPlay(ctx context.Context, req *v1.RealPlayRequest) (reply *v1.RealPlayReply, err error) {
	reply = &v1.RealPlayReply{}
	if req.ChannelNo == "" {
		return nil, errors.ParamsError
	}
	if parser.Conf.RealPlayUrl == "" {
		return nil, errors.ConfError
	}

	client := &http.Client{}
	timeOut := 30

	client.Timeout = time.Duration(timeOut) * time.Second

	hreq, err := http.NewRequest("GET", fmt.Sprintf("%s?chanpubid=%s&videourltype=rtmp&transport=udp", parser.Conf.RealPlayUrl, req.ChannelNo), nil)
	if err != nil {
		logger.Error("RealPlay",
			zap.String("err", err.Error()))
		return nil, err
	}

	resp, err := client.Do(hreq)
	if err != nil {
		logger.Error("RealPlay",
			zap.String("err", err.Error()))
		return nil, errors.VendorError
	}
	defer resp.Body.Close()

	result, err := ioutil.ReadAll(resp.Body)
	code := gjson.GetBytes(result, "errcode").Int()
	errmsg := gjson.GetBytes(result, "errdesc").String()
	if code != 0 {
		logger.Error("RealPlay",
			zap.String("err", errmsg))
		return nil, status.Error(20018, "三方错误:"+errmsg)
	}
	reply.Flvurl = gjson.GetBytes(result, "flvurl").String()
	reply.Hlsurl = gjson.GetBytes(result, "hlsurl").String()
	reply.Rtcurl = gjson.GetBytes(result, "rtcurl").String()
	reply.Rtmpurl = gjson.GetBytes(result, "rtmpurl").String()
	reply.Vediourl = gjson.GetBytes(result, "videourl").String()
	reply.Sessionid = gjson.GetBytes(result, "strsessionid").String()
	return reply, nil
}
