
package dust

import (
	"context"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"github.com/jaryhe/gopkgs/logger"
)

func LatestDustMonitorDataFromInflux(ctx context.Context, req *v1.LatestDustMonitorDataRequest)(reply *v1.LatestDustMonitorDataReply, err error) {
	if req.Sn == "" {
		// 获取一个扬尘设备
		da := model.DeviceAll{}
		where := map[string]interface{}{
			"ProjectId":req.ProjectId,
			"DeviceCode":model.DeviceTypeDust,
			"VerifyStatus":1,
			"DeviceState":1,
		}
		err = da.Find(database.DB(), where)
		if err != nil && err != gorm.ErrRecordNotFound{
			return nil, errors.DataBaseError
		}
		if da.ID == 0 {
			return &v1.LatestDustMonitorDataReply{Data:&v1.DustMonitorData{}}, nil
		}
		req.Sn = da.SN
	}
	array := []model.DustMonitorInfo{}
	sql := fmt.Sprintf("select * from \"%s\" order by time desc limit 1 tz('Asia/Shanghai')", req.Sn)
	_, err = model.QueryInfluxdb(sql, "DustMonitorInfo", &array)
	if err != nil {
		logger.Error("LatestDustMonitorData",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply = &v1.LatestDustMonitorDataReply{}
	if len(array) == 0 {
		return &v1.LatestDustMonitorDataReply{Data:&v1.DustMonitorData{}}, nil
	}
	reply = &v1.LatestDustMonitorDataReply{}
	reply.Data = &v1.DustMonitorData{}
	reply.Data.Date = array[0].Time
	reply.Data.B03 = array[0].Voice
	reply.Data.H01 = array[0].Humidity
	reply.Data.P01 = array[0].Atmospheric
	reply.Data.Pm10 = array[0].Pm10Value
	reply.Data.Pm25 = array[0].Pm25Value
	reply.Data.R01 = array[0].Rain
	reply.Data.T01 = array[0].Temperature
	reply.Data.Tsp = array[0].Tsp
	reply.Data.W01 = array[0].WindDirection
	reply.Data.W02 = array[0].WindSpeed
	return reply, nil


}

func LatestDustMonitorData(ctx context.Context, req *v1.LatestDustMonitorDataRequest)(reply *v1.LatestDustMonitorDataReply, err error) {

	return LatestDustMonitorDataFromInflux(ctx, req)
	/*
	p := model.DustMonitorInfo{}
	where := map[string]interface{}{
		"Type":consts.DustDataTypeMinute,
	}
	if req.Sn != "" {
		where["SN"] = req.Sn
	}
	err = p.FindSort(database.DB(), where, "MonitoringTime desc")
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return &v1.LatestDustMonitorDataReply{Data:&v1.DustMonitorData{}}, nil
	}
	reply = &v1.LatestDustMonitorDataReply{}
	reply.Data = &v1.DustMonitorData{}
	reply.Data.Date = p.MonitoringTime.Format("2006-01-02 15:04:05")
	reply.Data.B03 = p.Voice
	reply.Data.H01 = p.Humidity
	reply.Data.P01 = p.Atmospheric
	reply.Data.Pm10 = p.Pm10Value
	reply.Data.Pm25 = p.Pm25Value
	reply.Data.R01 = p.Rain
	reply.Data.T01 = p.Temperature
	reply.Data.Tsp = p.Tsp
	reply.Data.W01 = p.WindDirection
	reply.Data.W02 = p.WindSpeed
	return reply, nil

	 */
}
