
package lift

import (
	"context"
	"fmt"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"time"
)


func LatestLiftMonitorData(ctx context.Context, req *v1.LatestLiftMonitorDataRequest)(reply *v1.LatestLiftMonitorDataReply, err error) {

	if req.Sn == "" {
		return nil, errors.ParamsError
	}


	dbname := "lift_monitor_realtime_data"
	array := []LiftRealtimeData{}
	sql := fmt.Sprintf("select * from \"%s\" order by time desc limit 1 tz('Asia/Shanghai')", req.Sn)
	fmt.Printf("%s,%s\n", dbname, sql)
	_, err = model.QueryInfluxdb(sql, dbname, &array)
	if err != nil {
		logger.Error("LatestLiftMonitorData",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply = &v1.LatestLiftMonitorDataReply{}
	if len(array) == 0 {
		return &v1.LatestLiftMonitorDataReply{}, nil
	}

	reply.Date = time.Unix(array[0].DateTime, 0).Format("2006-01-02 15:04:05")
	reply.Speed = array[0].Speed
	reply.Weight = array[0].Weight
	reply.High = array[0].High
	reply.AngleX = array[0].Tilt1
	reply.AngleY = array[0].Tilt2
	if array[0].BackDoorState > 0 {
		reply.BackDoor = true
	}
	if array[0].FrontDoorState > 0 {
		reply.FrontDoor = true
	}
	reply.SpeedDirection = int32(array[0].Direction)
	reply.LeftRight = int32(array[0].LiftDirect)
	codes := getAlarmCodes(&array[0])
	for _, code := range codes {
		if reason, ok := LiftAlarmCodeReasonMap[code]; ok {
			reply.AlarmReasons = append(reply.AlarmReasons, reason)
		}
	}
	return reply, nil


}
