// Copyright 2019  githup.com. All rights reserved.
// Use of this source code is governed by  githup.com.
package project

import (
	"context"
	"encoding/base64"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/util"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-site-management/consts"
	"smart-site-management/errors"
	dbmodel "smart-site-management/model"
	pb_v1 "smart-site-management/pb/v1"
	"smart-site-management/pb"
	"time"
)

const CRYPTO_KEY = "2D29BP43U3Y1B4N6REQW3F319DD23454"

func makeDeviceKey(sn string, deviceType int32 ) (string) {
	cryPasswd, _ := util.AesEncrypt(fmt.Sprintf("%s%d", sn, deviceType), CRYPTO_KEY)
	ret := base64.StdEncoding.EncodeToString(cryPasswd)
	return ret
}

func postDustDeviceToProvincial(id int64, db *gorm.DB) error {

	dust := dbmodel.DustDeviceInfo{}
	where := map[string]interface{}{
		"ID":id,
	}
	err := dust.Find(db, where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return errors.DataBaseError
	}
	if dust.ID == 0 {
		return errors.DeviceNotExistError
	}
	mreq := pb_v1.ProvincialPostDustDeviceRequest{
		Code:dust.Code,
		Name:dust.Name,
		DeviceName:dust.DeviceName,
		DeviceId:dust.DeviceId,
		Manufacturer:dust.Manufacturer,
		Batch:dust.Batch,
		Unit:dust.Unit,
		DeviceModel:dust.DeviceModel,
	}
	_, err = pb.Thirdparty.ProvincialPostDustDevice(context.Background(), &mreq)
	if err != nil {
		return err
	}

	return nil
}

func dustDeviceApprove(db *gorm.DB, id int64, value map[string]interface{}) error {
	p := dbmodel.DustDeviceInfo{}
	where := map[string]interface{}{
		"ID":id,
		"VerifyStatus":consts.DeviceStatusAddAuditting,
	}
	return p.Update(db, where, value)
}

func attendanceDeviceApprove(db *gorm.DB, id int64, value map[string]interface{}) error {
	p := dbmodel.AttendanceDeviceInfo{}
	where := map[string]interface{}{
		"ID":id,
		"VerifyStatus":consts.DeviceStatusAddAuditting,
	}
	return p.Update(db, where, value)
}

func towerDeviceApprove(db *gorm.DB, id int64, value map[string]interface{}) error {
	p := dbmodel.TowerDeviceInfo{}
	where := map[string]interface{}{
		"ID":id,
		"VerifyStatus":consts.DeviceStatusAddAuditting,
	}
	return p.Update(db, where, value)
}

func liftDeviceApprove(db *gorm.DB, id int64, value map[string]interface{}) error {
	p := dbmodel.LiftDeviceInfo{}
	where := map[string]interface{}{
		"ID":id,
		"VerifyStatus":consts.DeviceStatusAddAuditting,
	}
	return p.Update(db, where, value)
}

func slagDeviceApprove(db *gorm.DB, id int64, value map[string]interface{}) error {
	p := dbmodel.SlagcarDevice{}
	where := map[string]interface{}{
		"ID":id,
		"VerifyStatus":consts.DeviceStatusAddAuditting,
	}
	return p.Update(db, where, value)
}

func vedioDeviceApprove(db *gorm.DB, id int64, value map[string]interface{}) error {
	p := dbmodel.Vssdevtbl{}
	where := map[string]interface{}{
		"ID":id,
		"VerifyStatus":consts.DeviceStatusAddAuditting,
	}
	delete(value, "Key")
	return p.Update(db, where, value)
}

func subDeviceApprove(db *gorm.DB, deviceCode uint32, id int64, value map[string]interface{}) error {
	switch deviceCode {
	case dbmodel.DeviceTypeDust:
		return dustDeviceApprove(db, id, value)
	case dbmodel.DeviceTypeSlag:
		return slagDeviceApprove(db, id, value)
	case dbmodel.DeviceTypeVedio:
		return vedioDeviceApprove(db, id, value)
	case dbmodel.DeviceTypeAttendance:
		return attendanceDeviceApprove(db, id, value)
	case dbmodel.DeviceTypeTower:
		return towerDeviceApprove(db, id, value)
	case dbmodel.DeviceTypeLift:
		return liftDeviceApprove(db, id, value)
	}
	return nil
}

func DeviceApprove(ctx context.Context, req *pb_v1.DeviceApproveRequest)(reply *pb_v1.DeviceApproveReply, err error) {
	p := &dbmodel.DeviceAll{}
	where := map[string]interface{}{
		"SubId":req.Id,
		"DeviceCode":req.DeviceCode,
	}
	// 状态判断
	err = p.Find(database.DB(), where)
	if err != nil {
		logger.Error("DeviceApprove",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.NoRecordError
	}
	if p.VerifyStatus != consts.DeviceStatusAddAuditting {
		return nil, errors.CanNotApproveWithStatus
	}

	// 更新
	where = map[string]interface{}{
		"SubId":req.Id,
		"DeviceCode":req.DeviceCode,
		"VerifyStatus":consts.DeviceStatusAddAuditting,
	}
	value := map[string]interface{}{
		"VerifyStatus":consts.DeviceStatusAddAuditted,
		"VerifyTime":time.Now(),
		"Key": makeDeviceKey(p.SN, int32(p.DeviceCode)),

	}
	if !req.Status {
		value["VerifyStatus"] = consts.DeviceStatusAddAudittedFail
	}
	db := database.DB().Begin()
	// 更新总表
	err = p.Update(db, where, value)
	if err != nil {
		logger.Error("DeviceApprove",
			zap.String("err", err.Error()))
		db.Rollback()
		return nil, errors.DataBaseError
	}

	if p.DeviceCode == dbmodel.DeviceTypeDust {
		// TODO 正式环境向省厅发数据
		if false {
			if err = postDustDeviceToProvincial(req.Id, db); err != nil {
				logger.Error("DeviceApprove",
					zap.String("err", err.Error()))
				db.Rollback()
				return nil, errors.ProvincialFailed
			}
		}

	}

	// 更新子表
	err = subDeviceApprove(db, p.DeviceCode, req.Id, value)
	if err != nil {
		logger.Error("DeviceApprove",
			zap.String("err", err.Error()))
		db.Rollback()
		return nil, errors.DataBaseError
	}
	db.Commit()
	return &pb_v1.DeviceApproveReply{}, nil
}

func DeviceApproveTest() {
	for i := 6; i <= 205; i++ {
		req := pb_v1.DeviceApproveRequest{}
		req.Id = int64(i)
		req.Status = true
		req.Feedback = "tongguo"
		req.DeviceCode = dbmodel.DeviceTypeDust
		_, err := DeviceApprove(context.Background(), &req)
		fmt.Printf("%v\n", err)
		if err != nil {
			break
		}
	}
}
