// Copyright 2019  githup.com. All rights reserved.
// Use of this source code is governed by  githup.com.
package project

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"smart-site-management/model"
	 "smart-site-management/pb/v1"
	"smart-site-management/consts"
	"strings"
)


func ProjectInfo(ctx context.Context, req *v1.ProjectInfoRequest)(reply *v1.ProjectInfoReply, err error) {
	if req.ProjectId < 1 {
		return nil, errors.ParamsError
	}
	where := map[string]interface{}{
		"ID": req.ProjectId,
	}
	p := model.ProjectInfo{}
	p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.NoRecordError
	}

	corp := model.ProjectCorpInfo{}
	where = map[string]interface{}{
		"ProjectId":req.ProjectId,
	}
	corpList, err := corp.All(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	pm := model.ProjectPMInfo{}
	where = map[string]interface{}{
		"PType":consts.StaffTypeManager,
	}
	pm.Find(database.DB(), where)




	reply = &v1.ProjectInfoReply{}
	reply.Id = p.ID
	reply.Location = p.Address
	reply.Name = p.Name
	reply.SafetyRecordNo = p.Safetyno
	reply.Lon = p.Lng
	reply.Lat = p.Lat
	reply.Price = p.Invest
	reply.CoveredArea = p.Buildingarea
	reply.Desc = p.Description
	reply.StartDay = p.Startdate.Format("2006-01-02")
	reply.EndDay = p.Enddate.Format("2006-01-02")
	reply.CreateTime = p.Createdat.Format("2006-01-02 15:04:05")
	reply.ProjectStatus = consts.ProjectStatusM[p.Prjstatus]
	reply.Category = consts.ProjectCategoryM[p.Category]
	reply.ConstructType = consts.ProjectConstructTypeM[p.Constructtype]

	if !strings.Contains(p.Planpic, "[") {
		reply.ProjectPlan = "[]"
	} else {
		reply.ProjectPlan = p.Planpic
	}
	if !strings.Contains(p.Effectpic, "[") {
		reply.ProjectEffectPic = "[]"
	} else {
		reply.ProjectEffectPic = p.Effectpic
	}
	for _, v := range corpList {
		if v.CorpType == consts.ProjectCorpDesign {
			reply.Design = v.CorpName
			continue
		}
		if v.CorpType == consts.ProjectCorpInvestigation {
			reply.Exploration = v.CorpName
			continue
		}
		if v.CorpType == consts.ProjectCorpSupervisor {
			reply.Oversee = v.CorpName
			continue
		}
		if v.CorpType == consts.ProjectCorpConstruction {
			reply.Development = v.CorpName
			continue
		}
		if v.CorpType == consts.ProjectCorpContract {
			reply.Construction = v.CorpName
			continue
		}
	}
	reply.Manager = pm.PMName
	reply.ManagerPhone = pm.PMPhone

	return reply, nil
}
