// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"google.golang.org/grpc/status"
	"smart-site-management/consts"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"time"
)

// 添加参建单位
func ProjectCorpAdd(ctx context.Context, req *v1.ProjectCorpAddRequest) (reply *v1.ProjectCorpAddReply, err error) {
	reply = &v1.ProjectCorpAddReply{}

	if req.ProjectId == 0 ||
		req.AreaCode == "" ||
		req.Name == "" ||
		req.SocialCode == "" ||
		req.CorpType == 0 ||
		req.RegisterDate == 0{
		return nil, errors.ParamsError
	}

	// 检查行政区域代码
	areaCode := model.AreaCode{}
	where := map[string]interface{}{
		"Code":req.AreaCode,
	}
	exist, err := areaCode.Count(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if exist == 0 {
		return nil, status.Error(10003, "行政区域代码不存在")
	}

	// 检查项目
	project := model.ProjectInfo{}
	where = map[string]interface{}{
		"ID":req.ProjectId,
	}
	exist, err = project.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if exist == 0 {
		return nil, errors.ProjectNotExistError
	}

	// 检查参建类型
	if _, ok := consts.ProjectCorpM[req.CorpType]; !ok {
		return nil,status.Error(10003, "不支持的参建类型")
	}

	// 检查是否已存在单位

	p := model.ProjectCorpInfo{}

	where = map[string]interface{}{
		"ProjectId":req.ProjectId,
		"CorpCode":req.SocialCode,
		"CorpType":req.CorpType,
	}
	exist, err = p.Count(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if exist > 0 {
		return nil, errors.CorpExist
	}

	p.CorpType = req.CorpType
	p.ProjectId = req.ProjectId
	p.AreaCode = req.AreaCode
	p.RegisterDate = time.Unix(req.RegisterDate, 0)
	p.Code = project.Code
	p.CorpCode = req.SocialCode
	p.CorpName = req.Name
	p.CreatedAt = time.Now()
	p.UpdatedAt = p.CreatedAt

	err = p.Insert(database.DB())
	if err != nil {
		return nil, errors.DataBaseError
	}

	return &v1.ProjectCorpAddReply{Id:p.ID}, nil

}
