// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"google.golang.org/grpc/status"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"time"
)

// 更新参建单位
func ProjectCorpUpdate(ctx context.Context, req *v1.ProjectCorpUpdateRequest) (reply *v1.ProjectCorpUpdateReply, err error) {
	reply = &v1.ProjectCorpUpdateReply{}

	if req.Id == 0 ||
		req.AreaCode == "" ||
		req.Name == "" ||
		req.SocialCode == "" ||
		req.CorpType == 0 ||
		req.RegisterDate == 0{
		return nil, errors.ParamsError
	}

	// 检查行政区域代码
	areaCode := model.AreaCode{}
	where := map[string]interface{}{
		"Code":req.AreaCode,
	}
	exist, err := areaCode.Count(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if exist == 0 {
		return nil, status.Error(10003, "行政区域代码不存在")
	}


	// 检查老数据
	old := model.ProjectCorpInfo{}
	where = map[string]interface{}{
		"ID":req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound{
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.CorpNotExist
	}
	if old.ProjectId != req.ProjectId {
		return nil, status.Error(10003, "项目和参建单位不匹配")
	}

	if req.CorpType != old.CorpType {
		pm := model.ProjectPMInfo{}
		where := map[string]interface{}{
			"CorpId":old.ID,
		}
		count, err := pm.Count(database.DB(), where)
		if err != nil {
			return nil, errors.DataBaseError
		}
		if count > 0 {
			return nil, status.Error(20031, "更改参建类型，请先删除已绑定的管理人员")
		}
	}



	// 检查是否已存在单位
	p := model.ProjectCorpInfo{}

	where = map[string]interface{}{
		"ProjectId":old.ProjectId,
		"CorpCode":req.SocialCode,
		"CorpType":req.CorpType,
		"ID !=":req.Id,
	}
	exist, err = p.Count(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if exist > 0 {
		return nil, errors.CorpExist
	}



	where = map[string]interface{}{
		"ID":req.Id,
	}
	values := map[string]interface{}{
		"AreaCode":req.AreaCode,
		"RegisterDate":time.Unix(req.RegisterDate, 0),
		"CorpCode":req.SocialCode,
		"CorpName":req.Name,
		"UpdatedAt":time.Now(),
		"CorpType":req.CorpType,
	}

	db := database.DB().Begin()
	err = p.Update(db, where, values)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	if old.CorpCode != req.SocialCode || old.CorpName != req.Name {
		pm := model.ProjectPMInfo{}
		where := map[string]interface{}{
			"CorpId":old.ID,
		}
		values := map[string]interface{}{
			"CorpName":req.Name,
			"CorpCode":req.SocialCode,
		}
		err := pm.Update(db, where, values)
		if err != nil {
			db.Rollback()
			return nil, errors.DataBaseError
		}
	}
	db.Commit()
	return &v1.ProjectCorpUpdateReply{}, nil

}
