// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"encoding/base64"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"google.golang.org/grpc/status"
	"smart-site-management/consts"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"github.com/jaryhe/gopkgs/util"
	"smart-site-management/utils"
	"time"
)

func certEncrypt(cert string) (string) {
	cryPasswd, _ := util.AesEncrypt(cert, CRYPTO_KEY)
	ret := base64.StdEncoding.EncodeToString(cryPasswd)
	return ret
}

func certDecrypt(cert string) (string) {
	bytes,_ := base64.StdEncoding.DecodeString(cert)
	ret, _ := util.AesDecrypt(bytes, []byte(CRYPTO_KEY))
	return string(ret)
}


func checkPtypeAndCtype(ptype uint32, ctype uint32) bool {
	switch ctype {
	case consts.ProjectCorpSupervisor :
		if ptype == consts.StaffTypeChiefSupervisor ||
			ptype == consts.StaffTypeDeputyChiefSupervisor ||
			ptype == consts.StaffTypeChiefSupervisorDelegate ||
			ptype == consts.StaffTypeProfessionalSupervisor ||
			ptype == consts.StaffTypeNormalSupervisor ||
			ptype == consts.StaffTypeSupervisor ||
			ptype == consts.StaffTypeSafetyChiefSupervisor ||
			ptype == consts.StaffTypeSupervisorSafty ||
			ptype == consts.StaffTypeOther {
			return true
		}
	case consts.ProjectCorpConstruction:
		if ptype == consts.StaffTypeBigEngineer ||
			ptype == consts.StaffTypeManager ||
			ptype == consts.StaffTypeEngineerManager ||
			ptype == consts.StaffTypeTechnical ||
			ptype == consts.StaffTypeEngineer {
			return true
		}

	case consts.ProjectCorpContract:
		if ptype == consts.StaffTypeManager ||
			ptype == consts.StaffTypeDeputyManager ||
			ptype == consts.StaffTypeChiefInspector ||
			ptype == consts.StaffTypeEngineer ||
			ptype == consts.StaffTypeAdmin ||
			ptype == consts.StaffTypeConstructor ||
			ptype == consts.StaffTypeQuality ||
			ptype == consts.StaffTypeSafety ||
			ptype == consts.StaffTypeStandard ||
			ptype == consts.StaffTypeMaterial ||
			ptype == consts.StaffTypeMachine ||
			ptype == consts.StaffTypeLabor ||
			ptype == consts.StaffTypeData ||
			ptype == consts.StaffTypeMeasure ||
			ptype == consts.StaffTypeTester ||
			ptype == consts.StaffTypeCost ||
			ptype == consts.StaffTypeQualityResponsible ||
			ptype == consts.StaffTypeSafetyResponsible ||
			ptype == consts.StaffTypeTechnicalResponsible {
			return true
		}
	default:
		if ptype == consts.StaffTypeManager ||
			ptype == consts.StaffTypeSafety ||
			ptype == consts.StaffTypeLabor ||
			ptype == consts.StaffTypeProduceManager ||
			ptype == consts.StaffTypeSourceManager ||
			ptype == consts.StaffTypeTechnicalResponsible ||
			ptype == consts.StaffTypeOther {
			return true
		}

	}
	return false
}

// 添加管理人员
func ProjectPMAdd(ctx context.Context, req *v1.ProjectPMAddRequest) (reply *v1.ProjectPMAddReply, err error) {
	reply = &v1.ProjectPMAddReply{}

	if req.ProjectId == 0 ||
		req.Name == "" ||
		req.CorpId == 0 ||
		req.Phone == "" ||
		req.CertType == 0 ||
		req.CertNum == "" ||
		req.PType == 0 {
		return nil, errors.ParamsError
	}
	// 检查证件类型
	if _, ok := consts.CertTypeM[req.CertType];!ok {
		return nil, status.Error(10003, "不支持的证件类型")
	}
	if _, ok := utils.CheckIDCert(req.CertNum); !ok && req.CertType == consts.CertTypeID {
		return nil, status.Error(10003, "身份证格式错误")
	}
	if !utils.VerifyMobileFormat(req.Phone) {
		return nil, status.Error(10003, "手机号格式错误")
	}

	// 检查项目
	project := &model.ProjectInfo{}
	where := map[string]interface{}{
		"ID":req.ProjectId,
	}
	exist, err := project.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if exist == 0 {
		return nil, errors.ProjectNotExistError
	}


	// 检查人员类型
	if _, ok := consts.StaffTypeM[req.PType]; !ok {
		return nil,status.Error(10003, "不支持的人员类型")
	}

	// 检查所属企业信息
	corp := &model.ProjectCorpInfo{}
	where = map[string]interface{}{
		"ID":req.CorpId,
		"ProjectId":req.ProjectId,
	}
	err = corp.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if corp.ID == 0 {
		return nil, errors.CorpNotExist
	}

	if !checkPtypeAndCtype(req.PType, corp.CorpType) {
		return nil, errors.PmCorpNotMatch
	}

	// 检查是否已存在人员
	p := &model.ProjectPMInfo{}

	aesCert := certEncrypt(req.CertNum)
	where = map[string]interface{}{
		"ProjectId":req.ProjectId,
		"PMIDCardNumber":aesCert,
	}
	exist, err = p.Count(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if exist > 0 {
		return nil, errors.PMExist
	}

	p.CorpType = corp.CorpType
	p.ProjectId = req.ProjectId
	p.Code = project.Code
	p.CorpCode = corp.CorpCode
	p.CorpName = corp.CorpName
	p.PMName = req.Name
	p.PMIDCardType = req.CertType
	p.PMIDCardNumber = aesCert
	p.PType = req.PType
	p.PMPhone = req.Phone
	p.CorpId = req.CorpId
	p.CreatedAt = time.Now()
	p.UpdatedAt = p.CreatedAt

	err = p.Insert(database.DB())
	if err != nil {
		return nil, errors.DataBaseError
	}

	return &v1.ProjectPMAddReply{Id:p.ID}, nil

}
