// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
)

// 管理人员列表
func ProjectPMList(ctx context.Context, req *v1.ProjectPMListRequest) (reply *v1.ProjectPMListReply, err error) {
	reply = &v1.ProjectPMListReply{}

	if req.ProjectId == 0 {
		return nil, errors.ParamsError
	}
	if req.Page < 1 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}

	p := model.ProjectPMInfo{}
	where := map[string]interface{}{
		"ProjectId":req.ProjectId,
	}
	if req.Filter != "" {
		where["PMName like"] = "%"+req.Filter+"%"
	}
	if req.CorpId > 0 {
		where["CorpId"] = req.CorpId
	}
	count, err := p.Count(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound{
		return nil, errors.DataBaseError
	}
	reply.Total = count
	reply.PageSize = req.PageSize
	reply.Page = req.Page
	if count == 0 {
		return reply, nil
	}

	list, err := p.List(database.DB(), where, req.Page, req.PageSize)
	if err != nil && err != gorm.ErrRecordNotFound{
		return nil, errors.DataBaseError
	}
	reply.List = make([]*v1.ProjectPMListItem, len(list))


	for i, v := range list {
		plainCert := certDecrypt(v.PMIDCardNumber)
		item := &v1.ProjectPMListItem{
			CorpType:v.CorpType,
			CorpName:v.CorpName,
			Id:v.ID,
			SocialCode:v.CorpCode,
			CertType:v.PMIDCardType,
			PType:v.PType,
			Name:v.PMName,
			Phone:v.PMPhone,
			CertNum:plainCert,
			CorpId:v.CorpId,
			CreatedTime:v.CreatedAt.Format(layout),
		}
		reply.List[i] = item
	}

	return reply, nil
}
