// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"google.golang.org/grpc/status"
	"smart-site-management/consts"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"smart-site-management/utils"
	"time"
)


// 添加管理人员
func ProjectPMUpdate(ctx context.Context, req *v1.ProjectPMUpdateRequest) (reply *v1.ProjectPMUpdateReply, err error) {
	reply = &v1.ProjectPMUpdateReply{}

	if req.Id == 0 ||
		req.Name == "" ||
		req.CorpId == 0 ||
		req.Phone == "" ||
		req.CertType == 0 ||
		req.CertNum == "" ||
		req.PType == 0 {
		return nil, errors.ParamsError
	}
	// 检查证件类型
	if _, ok := consts.CertTypeM[req.CertType];!ok {
		return nil, status.Error(10003, "不支持的证件类型")
	}
	if _, ok := utils.CheckIDCert(req.CertNum); !ok && req.CertType == consts.CertTypeID {
		return nil, status.Error(10003, "身份证格式错误")
	}
	if !utils.VerifyMobileFormat(req.Phone) {
		return nil, status.Error(10003, "手机号格式错误")
	}


	// 检查人员类型
	if _, ok := consts.StaffTypeM[req.PType]; !ok {
		return nil,status.Error(10003, "不支持的人员类型")
	}


	// 检查所属企业信息
	corp := &model.ProjectCorpInfo{}
	where := map[string]interface{}{
		"ID":req.CorpId,
		"ProjectId":req.ProjectId,
	}
	err = corp.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if corp.ID == 0 {
		return nil, errors.CorpNotExist
	}

	if !checkPtypeAndCtype(req.PType, corp.CorpType) {
		return nil, errors.PmCorpNotMatch
	}

	// 检查老数据
	old := model.ProjectPMInfo{}
	where = map[string]interface{}{
		"ID":req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound{
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.NoRecordError
	}
	if old.ProjectId != req.ProjectId {
		return nil, status.Error(10003, "项目和人员不匹配")
	}

	// 检查是否已存在
	p := model.ProjectPMInfo{}

	aesCert := certEncrypt(req.CertNum)
	where = map[string]interface{}{
		"ProjectId":old.ProjectId,
		"PMIDCardNumber":aesCert,
		"ID !=":req.Id,
	}
	exist, err := p.Count(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if exist > 0 {
		return nil, errors.PMExist
	}

	where = map[string]interface{}{
		"ID":req.Id,
	}
	values := map[string]interface{}{
		"CorpType":corp.CorpType,
		"CorpName":corp.CorpName,
		"CorpCode":corp.CorpCode,
		"CorpId":corp.ID,
		"PMName":req.Name,
		"PMIDCardType":req.CertType,
		"PMIDCardNumber":aesCert,
		"PType":req.PType,
		"PMPhone":req.Phone,
		"UpdatedAt":time.Now(),
	}


	err = p.Update(database.DB(), where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}

	return &v1.ProjectPMUpdateReply{}, nil

}
