// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
)

// 添加备案车辆
func ProjectVehicleAdd(ctx context.Context, req *v1.ProjectVehicleAddRequest) (reply *v1.ProjectVehicleAddReply, err error) {
	reply = &v1.ProjectVehicleAddReply{}

	if req.ProjectId == 0 ||
		req.PlateNo == "" {
		return nil, errors.ParamsError
	}


	// 检查项目
	project := model.ProjectInfo{}
	where := map[string]interface{}{
		"ID":req.ProjectId,
	}
	exist, err := project.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if exist == 0 {
		return nil, errors.ProjectNotExistError
	}

	// 检查是否已存在车辆
	p := model.VehicleRecord{}
	where = map[string]interface{}{
		"ProjectId":req.ProjectId,
		"PlateNo":req.PlateNo,
	}
	exist, err = p.Count(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if exist > 0 {
		return nil, errors.VehicleExist
	}

	p.PlateNo = req.PlateNo
	p.ProjectId = req.ProjectId

	err = p.Insert(database.DB())
	if err != nil {
		return nil, errors.DataBaseError
	}

	return &v1.ProjectVehicleAddReply{Id:p.ID}, nil

}
