// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
)

// 备案车辆列表
func ProjectVehicleList(ctx context.Context, req *v1.ProjectVehicleListRequest) (reply *v1.ProjectVehicleListReply, err error) {
	reply = &v1.ProjectVehicleListReply{}

	if req.ProjectId == 0 {
		return nil, errors.ParamsError
	}
	if req.Page < 1 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}

	p := model.VehicleRecord{}
	where := map[string]interface{}{
		"ProjectId":req.ProjectId,
	}
	count, err := p.Count(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound{
		return nil, errors.DataBaseError
	}
	reply.Total = count
	reply.PageSize = req.PageSize
	reply.Page = req.Page
	if count == 0 {
		return reply, nil
	}

	list, err := p.List(database.DB(), where, req.Page, req.PageSize)
	if err != nil && err != gorm.ErrRecordNotFound{
		return nil, errors.DataBaseError
	}
	reply.List = make([]*v1.ProjectVehicleListItem, len(list))


	for i, v := range list {
		item := &v1.ProjectVehicleListItem{
			Id:v.ID,
			PlateNo:v.PlateNo,
		}
		reply.List[i] = item
	}

	return reply, nil
}
