// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package staff

import (
	"context"
	"encoding/base64"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"smart-site-management/consts"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"smart-site-management/utils"
	"github.com/jaryhe/gopkgs/util"
)

const CRYPTO_KEY = "2D29BP43U3Y1B4N6REQW3F319DD23454"

func certEncrypt(cert string) string {
	cryPasswd, _ := util.AesEncrypt(cert, CRYPTO_KEY)
	ret := base64.StdEncoding.EncodeToString(cryPasswd)
	return ret
}

func certDecrypt(cert string) string {
	bytes, _ := base64.StdEncoding.DecodeString(cert)
	ret, _ := util.AesDecrypt(bytes, []byte(CRYPTO_KEY))
	return string(ret)
}

func StaffList(ctx context.Context, req *v1.StaffListRequest) (reply *v1.StaffListReply, err error) {
	reply = &v1.StaffListReply{}
	if req.ProjectId == 0 {
		return nil,errors.ParamsError
	}
	if req.Page < 1 {
		req.Page =1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	p := &model.TStaff{}
	list, total, err := p.List(database.DB(), req.Filter, req.ProjectId, req.Page, req.PageSize)
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.Total = total
	reply.PageSize = req.PageSize
	reply.Page = req.Page
	if total == 0 {
		return reply, nil
	}

	reply.List = make([]*v1.StaffListItem, len(list))
	workTypeMap, _ := model.GetWorkItemMap()
	for i, v := range list {
		item := &v1.StaffListItem{}
		utils.StructCopy(item, &list[i], "")
		item.Id = v.Id
		item.CreatedAt = v.CreatedAt.Format("2006-01-02 15:04:05")
		item.WorkType = workTypeMap[v.WorkType]
		item.StaffType = consts.StaffTypeM[uint32(v.StaffType)]
		item.Gender = v.Gender
		item.WorkNo = v.WorkNo
		item.Group = v.TeamName
		item.ProjectId = v.ProjectId
		item.Address = v.Address
		item.IdCert = certDecrypt(v.IDCardNumber)
		item.IdIssue = "身份证"
		item.Sn = v.Sn
		if v.IDCardType != 1 {
			item.IdIssue = "护照"
		}
		item.Name = v.WorkName
		item.Birthday = fmt.Sprintf("%d", v.Age)
		item.Photo = v.HeadImage
		item.LaborCompany = v.CorpName
		reply.List[i] = item
	}
	return reply, nil
}
