// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package staff

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
)

func StaffWorkTypeStatistics(ctx context.Context, req *v1.StaffWorkTypeStatisticsRequest) (reply *v1.StaffWorkTypeStatisticsReply, err error) {
	reply = &v1.StaffWorkTypeStatisticsReply{}

	if req.ProjectId == 0 {
		return nil,errors.ParamsError
	}

	list, err := model.StaffWorkTypeStaticList(database.DB(), req.ProjectId)
	if err != nil {
		logger.Error("StaffWorkTypeStaticList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	reply.List = make([]*v1.WorkTypeStatistics, len(list))
	for i, v := range list{
		item := &v1.WorkTypeStatistics{
			WorkType:v.TypeName,
			Total:int64(v.StaffCount),
			Attendance:int64(v.StaffAttendanceCount),
			Id:v.Id,
		}
		reply.List[i] = item
	}
	return reply, nil
}
