package task

import (
	"context"
	"encoding/json"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"time"
)

const (
TaskStatusUnProcessing = 1
TaskStatusUnApprove = 2
TaskStatusApprovedRefuse = 3
TaskStatusProcessed = 4
)

func TaskFinish(ctx context.Context, req *v1.TaskFinishRequest)(reply *v1.TaskFinishReply, err error) {
	if req.Id < 1  || req.ProjectId < 1 || req.ProcessingPerson == ""{
		return nil, errors.ParamsError
	}
	task := model.TaskProcessingInfo{}
	where := map[string]interface{}{
		"ID":req.Id,
		"ProjectID":req.ProjectId,
	}
	err = task.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if task.ID == 0 {
		return nil, errors.NoRecordError
	}

	if task.Status == TaskStatusProcessed {
		return nil, errors.CanNotApproveWithStatus
	}
	if len(req.ProcessingResultsImg) == 0 {
		req.ProcessingResultsImg = []string{}
	}
	img, _ := json.Marshal(req.ProcessingResultsImg)
	now := time.Now()
	values := map[string]interface{}{
		"Status":TaskStatusUnApprove,
		"ProcessingDate":now,
		"ProcessingResultsImg":string(img),
		"ProcessingResults":req.ProcessingResults,
		"ProcessingPerson":req.ProcessingPerson,
	}

	pipeArray := []model.TaskPipeline{}
	json.Unmarshal([]byte(task.Pipeline), &pipeArray)

	pipe := model.TaskPipeline{
		PipeType:model.PipelineProcessing,
		PipeTime:now.Format("2006-01-02 15:04:05"),
		Person:req.ProcessingPerson,
		Desc:req.ProcessingResults,
		Img:string(img),
		Status:"待审核",
	}
	if pipe.Img == "" {
		pipe.Img = "[]"
	}
	pipeArray = append(pipeArray, pipe)
	pipeBytes, _ := json.Marshal(pipeArray)
	values["Pipeline"] = string(pipeBytes)

	p := model.TaskProcessingInfo{}

	err = p.Update(database.DB(), where, values)

	if err != nil {
		return nil, errors.DataBaseError
	}

	return &v1.TaskFinishReply{}, nil
}
