package task

import (
	"context"
	"encoding/json"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
)


func TaskInfo(ctx context.Context, req *v1.TaskInfoRequest)(reply *v1.TaskInfoReply, err error) {
	if req.Id < 1 {
		return nil, errors.ParamsError
	}
	task := model.TaskProcessingInfo{}
	where := map[string]interface{}{
		"ID":req.Id,
	}
	err = task.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if task.ID == 0 {
		return nil, errors.NoRecordError
	}
	array := []model.TaskPipeline{}
	json.Unmarshal([]byte(task.Pipeline), &array)
	reply = &v1.TaskInfoReply{
		TaskName:task.TaskName,
		TaskCode:task.TaskID,
		TaskType:task.TaskType,
		Status:task.Status,
		FinishDate:task.FinishDate.Format("2006-01-02"),
	}
	reply.List = make([]*v1.TaskPipeline, len(array))
	for i, v := range array {
		item := &v1.TaskPipeline{
			Person:v.Person,
			PipeTime:v.PipeTime,
			PipeType:v.PipeType,
			Desc:v.Desc,
			Status:v.Status,
			Img:v.Img,
		}
		reply.List[i] = item
	}

	return reply, nil
}
