package task

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"time"
)


func TaskList(ctx context.Context, req *v1.TaskListRequest)(reply *v1.TaskListReply, err error) {

	if req.ProjectId < 1 {
		return nil, errors.ParamsError
	}
	if req.Page < 1 {
		req.Page = 1
	}
	if req.PageSize < 1 {
		req.PageSize = 10
	}
	now := time.Now()
	mreq := model.TaskListRequest{
		Start:req.Start,
		End:req.End,
		ProjectId:req.ProjectId,
		Filter:req.Filter,
		PageSize:req.PageSize,
		Page:req.Page,
		Status:req.Status,
		TaskType:req.TaskType,
		Overdue:req.Overdue,
		Source:req.Source,
		Now:now,
	}
	list, total, err := model.TaskList(database.DB(), &mreq)
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply = &v1.TaskListReply{
		Total:total,
		PageSize:req.PageSize,
		Page:req.Page,
	}
	if total == 0 {
		return reply, nil
	}
	reply.List = make([]*v1.TaskListItem, len(list))

	for i, v := range list {
		item := &v1.TaskListItem{
			Id:v.ID,
			ProjectId:v.ProjectID,
			TaskCode:v.TaskID,
			CreatedTime:v.AssignmentDate.Format("2006-01-02 15:04:05"),
			FinishDate:v.FinishDate.Format("2006-01-02"),
			ProcessingPerson:v.ProcessingPerson,
			ProcessingResults:v.ProcessingResults,
			ProcessingResultsImg:v.ProcessingResultsImg,
			Reviewer:v.Reviewer,
			Status:v.Status,
			ProjectName:v.ProjectName,
			SafetyRecordNo:v.SafetyRecordNo,
			TaskType:v.TaskType,
			TaskName:v.TaskName,
			TaskImg:v.TaskImg,
			Creater:v.Creater,
		}
		if now.Unix() > v.FinishDate.Unix() && v.ProcessingDate.Unix() <= 0{
			item.IsOverdue = true
		}
		if v.ProcessingDate.Unix() > v.FinishDate.Unix() {
			item.IsOverdue = true
		}
		if v.TaskImg == "" {
			item.TaskImg = "[]"
		}
		if v.ProcessingResultsImg == "" {
			item.ProcessingResultsImg = "[]"
		}
		reply.List[i] = item
	}

	return reply, nil
}
