
package tower

import (
	"context"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
)

func TowerBaseInfoList(ctx context.Context, req *v1.TowerBaseInfoListRequest)(reply *v1.TowerBaseInfoListReply, err error) {
	if req.ProjectId < 1 {
		return nil, errors.ParamsError
	}
	reply = &v1.TowerBaseInfoListReply{}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}
	reply.Page = req.Page
	reply.PageSize = req.PageSize
	where := map[string]interface{}{
		"ProjectID":req.ProjectId,
	}
	p := &model.TowerBaseInfo{}
	total, err := p.Count(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.Total = total
	if total == 0 {
		return reply, nil
	}
	list, err := p.List(database.DB(), where, req.PageSize, (req.Page-1)*req.PageSize)
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.List = make([]*v1.TowerBaseInfoItem, len(list))
	for i, v := range list {
		reply.List[i] = &v1.TowerBaseInfoItem{
			FrontArmLength:v.FrontArmLength,
			BackArmLength:v.BackArmLength,
			HatHigh:v.HatHigh,
			ArmHigh:v.ArmHigh,
			X:v.X,
			Y:v.Y,
			MaxWeight:float64(v.MaxWeight),
			MaxMoment:v.MaxMoment,
			Sn:v.SN,
			TowerNo:fmt.Sprintf("%d#", v.TowerNo),
			Id:v.ID,
		}
	}

	return reply, nil

}
