package tower

import (
	"context"
	"go.uber.org/zap"
	"smart-site-management/model"
	"fmt"
	"smart-site-management/errors"
	"smart-site-management/pb/v1"
	"github.com/jaryhe/gopkgs/logger"
	"strconv"
	"time"
)

var TowerAlarmValueReasonMap = map[int64]string{
	111: "碰撞报警",
	12:  "重量报警",
	13:  "力矩报警",
	14:  "倾斜报警",
	15:  "风速报警",
	101: "幅度限位报警",
	102: "高度限位报警",
	103: "回转限位报警",

	211: "碰撞预警",
	22:  "重量预警",
	23:  "力矩预警",
	24:  "倾斜预警",
	25:  "风速预警",
	201: "幅度限位预警",
	202: "高度限位预警",
	203: "回转限位预警",
}

func getTowerHistorySql(req *v1.TowerMonitorHistoryRequest) (string, string, string) {
	//layout := "2006-01-02 15:04:05"
	sql := fmt.Sprintf("select * from \"%s\" ", req.Sn)
	where := ""
	whereArray := []string{}
	if req.Start > 0 {
		whereArray = append(whereArray, fmt.Sprintf("date_time >= %d ", req.Start))
	}
	if req.End > 0 {
		whereArray = append(whereArray, fmt.Sprintf("date_time < %d ", req.End))
	}
	if req.IsAlarm {
		whereArray = append(whereArray, "alarm_reason > 0")
	}
	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf("where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}

	offset := int64(req.Page - 1) * int64(req.PageSize)
	countSql := fmt.Sprintf("select count(backturn) as count from \"%s\" %s tz('Asia/Shanghai')", req.Sn, where)
	sql = fmt.Sprintf("%s %s order by time limit %d offset %d tz('Asia/Shanghai')", sql, where, req.PageSize, offset)
	fmt.Printf("sql:%s\n", sql)
	return sql, countSql, "tower_monitor_realtime_data"
}

func getTowerHistoryCount(dbname string, sql string) (int64, error) {
	type Result struct {
		Count int64
	}
	array := []Result{}
	_, err := model.QueryInfluxdb(sql, dbname, &array)
	if err != nil {
		return 0, errors.DataBaseError
	}
	if len(array) == 0 {
		return 0, nil
	}
	return array[0].Count, nil
}

func floatParse(data float64) float64 {
	str := strconv.FormatFloat(data, 'f', 2, 64)
	ret, _ := strconv.ParseFloat(str, 64)
	return ret
}

func TowerMonitorHistory(ctx context.Context, req *v1.TowerMonitorHistoryRequest) (reply *v1.TowerMonitorHistoryReply, err error) {
	if req.Sn == "" {
		return nil, errors.ParamsError
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}


	reply = &v1.TowerMonitorHistoryReply{
		Page:req.Page,
		PageSize:req.PageSize,
	}

	sql, countSql, dbname := getTowerHistorySql(req)
	if err != nil {
		logger.Error("DustMonitorHistory",
			zap.String("err", err.Error()))
		return nil, err
	}
	total, err := getTowerHistoryCount(dbname, countSql)
	if err != nil {
		logger.Error("DustMonitorHistory",
			zap.String("err", err.Error()))
		return nil, err
	}
	reply.Total = total
	if reply.Total == 0 {
		return reply, nil
	}

	array := []TowerInfo{}
	_, err = model.QueryInfluxdb(sql, dbname, &array)
	if err != nil {
		logger.Error("TowerMonitorHistory",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}


	for i, v := range array {
		item := v1.TowerMonitorData{}
		item.Date = time.Unix(v.DateTime, 0).Format("2006-01-02 15:04:05")
		item.Angle = floatParse(array[i].Tilt)
		item.WindSpeed = floatParse(array[i].WindSpeed)
		item.Weight = float64(array[i].Weight)
		item.Hight = array[i].High
		item.Scope = floatParse(array[i].Range)
		item.BackTurn = array[i].Backturn
		item.Moment = int64(array[i].MomentPercent)
		item.AlarmReason = TowerAlarmValueReasonMap[v.AlarmReason]
		reply.List = append(reply.List, &item)
	}

	return reply, nil
}
