
package tower

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-site-management/model"
	"fmt"
	"smart-site-management/errors"
	"smart-site-management/pb/v1"
	"github.com/jaryhe/gopkgs/logger"
	"time"
)

type TowerInfo struct {
	DateTime int64 `json:"date_time"`
	// 回转角度值
	Backturn float64 `json:"backturn"`
	// 幅度值
	Range float64 `json:"range"`
	// 吊钩离地面的距离
	High float64 `json:"high"`
	// 吊钩所掉重物的重量
	Weight int64 `json:"weight"`
	// 当前力矩所占最大力矩的百分比
	MomentPercent int `json:"moment_percent"`
	// 风速
	WindSpeed float64 `json:"wind_speed"`
	// 塔身倾斜
	Tilt float64 `json:"tilt"`
	AlarmReason int64 `json:"alarm_reason"`
}



func getTowerDbName(tail string) string {
	return "tower_monitor_"+tail
}

func TowerBaseInfo(sn string)(*v1.TowerDeviceBaseInfo, error) {
	p := &model.TowerBaseInfo{}
	where := map[string]interface{}{
		"SN":sn,
	}
	err := p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound{
		return nil, errors.DataBaseError
	}
	ret := &v1.TowerDeviceBaseInfo{
		FrontArmLength:p.FrontArmLength,
		BackArmLength:p.BackArmLength,
		HatHigh:p.HatHigh,
		ArmHigh:p.ArmHigh,
	}
	return ret, nil
}
func LatestTowerMonitorData(ctx context.Context, req *v1.LatestTowerMonitorDataRequest)(reply *v1.LatestTowerMonitorDataReply, err error) {

	if req.Sn == "" {
		return nil, errors.ParamsError
	}


	dbname := getTowerDbName("realtime_data")
	array := []TowerInfo{}
	sql := fmt.Sprintf("select * from \"%s\" order by time desc limit 1 tz('Asia/Shanghai')", req.Sn)
	fmt.Printf("%s,%s\n", dbname, sql)
	_, err = model.QueryInfluxdb(sql, dbname, &array)
	if err != nil {
		logger.Error("LatestDustMonitorData",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply = &v1.LatestTowerMonitorDataReply{}
	if len(array) == 0 {
		return &v1.LatestTowerMonitorDataReply{Data:&v1.TowerMonitorData{}, Base:&v1.TowerDeviceBaseInfo{}}, nil
	}
	base, err := TowerBaseInfo(req.Sn)
	if err != nil {
		return nil, err
	}
	reply.Base = base
	reply.Data = &v1.TowerMonitorData{}
	reply.Data.Date = time.Unix(array[0].DateTime, 0).Format("2006-01-02 15:04:05")
	reply.Data.Angle = array[0].Tilt
	reply.Data.WindSpeed = array[0].WindSpeed
	reply.Data.Weight = float64(array[0].Weight)
	reply.Data.Hight = array[0].High
	reply.Data.Scope = array[0].Range
	reply.Data.BackTurn = array[0].Backturn
	reply.Data.Moment = int64(array[0].MomentPercent)
	return reply, nil


}
