// Copyright 2019  githup.com. All rights reserved.
// Use of this source code is governed by  githup.com.
package user

import (
	"context"
	"encoding/base64"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/util"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"smart-site-management/consts"
	"smart-site-management/errors"
	dbmodel "smart-site-management/model"
	pb_v1 "smart-site-management/pb/v1"
)

const CRYPTO_KEY = "3D28BP43U3Y1B4N6REQW3F319DD23467"

func getB64Pass(passwd string) (string, error) {
	cryPasswd, err := util.AesEncrypt(passwd, CRYPTO_KEY)
	if err != nil {
		return "", errors.ParamsError
	}
	ret := base64.StdEncoding.EncodeToString(cryPasswd)
	return ret, nil
}

func LoginByProjectId(ctx context.Context, req *pb_v1.LoginRequest)(reply *pb_v1.LoginReply, err error) {
	p := dbmodel.TSiteUser{}
	where := map[string]interface{}{"ProjectId":req.ProjectId, "UserType":consts.ProjectUserTypeProject}
	err = p.Find(database.DB(), where, nil)
	if err != nil  {
		logger.Error("Login",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.Id == 0 {
		return nil, errors.UserNotExistError
	}
	if !p.Enable  {
		return nil, status.Error(20009, "用户已禁用")
	}
	isProjectUser := false
	reply = &pb_v1.LoginReply{
		Uid:p.Id,
		ProjectUser:isProjectUser,
		ProjectId:p.ProjectId,
		UserName:p.Username,
	}
	return reply, nil
}

func Login(ctx context.Context, req *pb_v1.LoginRequest)(reply *pb_v1.LoginReply, err error) {
	if req.ProjectId > 0 {
		return LoginByProjectId(ctx, req)
	}
	if req.User == "" || req.Password == "" {
		return nil, errors.ParamsError
	}
	p := dbmodel.TSiteUser{}
	where := map[string]interface{}{"UserName":req.User}
	err = p.Find(database.DB(), where, nil)
	if err != nil  {
		logger.Error("Login",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.Id == 0 {
		return nil, errors.UserNotExistError
	}
	if !p.Enable  {
		return nil, status.Error(20009, "用户已禁用")
	}
	isProjectUser := true
	if p.UserType != consts.ProjectUserTypeProject {
		isProjectUser = false
		//return nil, status.Error(20009, "非项目用户无法访问")
	}
	pass,_ := getB64Pass(req.Password)
	if pass != p.Passwd {
		return nil, errors.UserPassError
	}

	reply = &pb_v1.LoginReply{
		Uid:p.Id,
		ProjectUser:isProjectUser,
		ProjectId:p.ProjectId,
		UserName:p.Username,
	}
	return reply, nil
}
