// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package vehicle

import (
	"context"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
	"time"
	"github.com/jaryhe/gopkgs/logger"
)

type VehicleInfo struct {
	PlateNo string `json:"plate_no"`
	Type int `json:"type"`
	Image string `json:"image"`
	Sn string `json:"sn"`
	CleanImage string `json:"clean_image"`
	VehicleType int32 `json:"vehicle_type"`
	Time string `json:"time"`
}

func VehicleListFromInflux(ctx context.Context, req *v1.VehicleListRequest) (reply *v1.VehicleListReply, err error) {
	if req.ProjectId == 0 {
		return nil, errors.ParamsError
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}
	reply = &v1.VehicleListReply{
		Page:req.Page,
		PageSize:int32(req.PageSize),
	}
	offset := int((req.Page - 1)) *int(req.PageSize)

	whereArray := []string{}
	if req.Start > 0 {
		whereArray = append(whereArray, fmt.Sprintf("time >= %ds", req.Start))
	}
	if req.End > 0 {
		whereArray = append(whereArray, fmt.Sprintf("time < %ds", req.End))
	}
	if req.PlateNo != "" {
		whereArray = append(whereArray, fmt.Sprintf("carNumber=~/%s/", req.PlateNo))
	}
	where := ""
	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf("where %s", v)
		} else {
			where = fmt.Sprintf("%s %s", where, v)
		}
	}
	// prepare sql
	sql :=  fmt.Sprintf("select * from \"project_%d\"  %s order by time desc limit %d offset %d tz('Asia/Shanghai')",req.ProjectId,  where, req.PageSize, offset)
	countSql := fmt.Sprintf("select count(carModel) as count from \"project_%d\" %s tz('Asia/Shanghai')",req.ProjectId, where)

	type Result struct {
		Count int64
	}
	array := []Result{}
	_, err = model.QueryInfluxdb(countSql, "Slagcar", &array)
	if err != nil {
		logger.Error("VehicleList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if len(array) == 0 {
		return  reply, nil
	}
	reply.Total = array[0].Count

	// get list
	lArray := []model.Slagcar{}
	_, err = model.QueryInfluxdb(sql, "Slagcar", &lArray)

	if err != nil {
		logger.Error("VehicleList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if len(lArray) == 0 {
		return  reply, nil
	}

	reply.List = make([]*v1.Vehicle, len(lArray))

	for i, v := range lArray {
		item := &v1.Vehicle{}
		item.Id = v.ID
		item.PlateNo = v.Carnumber
		item.CleanImage = v.Cleanimg
		item.Image = v.Carnumimg
		item.Date = v.Time
		item.Sn = v.Sn
		item.Type = "出"
		if v.Inout == 1 {
			item.Type = "进"
		}
		if v.Carmodel == 0 {
			item.VehicleType = "运渣车"
		} else if v.Carmodel == 1 {
			item.VehicleType = "罐车"
		} else {
			item.VehicleType = "其他"
		}
		reply.List[i] = item
	}
	return reply, nil

}

func VehicleList(ctx context.Context, req *v1.VehicleListRequest) (reply *v1.VehicleListReply, err error) {
	return VehicleListFromInflux(ctx, req)
	if req.ProjectId == 0 {
		return nil, errors.ParamsError
	}

	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}
	reply = &v1.VehicleListReply{
		Page:req.Page,
		PageSize:int32(req.PageSize),
	}

	p := model.Slagcar{}
	where := map[string]interface{}{
		"ProjectId": req.ProjectId,
		"carModel":0,
	}
	if req.Type > 2 {
		return nil, errors.ParamsError
	}
	if req.Type > 0 {
		where["carModel"] = req.Type
	}
	if req.Start > 0 {
		where["snapDate >="] = time.Unix(req.Start, 0)
	}
	if req.End > 0 {
		where["snapDate <"] = time.Unix(req.End, 0)
	}
	if req.PlateNo != "" {
		where["carNumber like"] = fmt.Sprintf("%%%s%%", req.PlateNo)
	}
	count, err := p.Count(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.Total = count
	if count == 0 {
		return reply, nil
	}
	list, err := p.List(database.DB(), where, req.Page, req.PageSize)
	if err != nil {
		return nil, errors.DataBaseError
	}

	reply.List = make([]*v1.Vehicle, len(list))
	for i, v := range list {
		item := &v1.Vehicle{}
		item.Id = v.ID
		item.PlateNo = v.Carnumber
		item.CleanImage = v.Cleanimg
		item.Image = v.Carnumimg
		item.Date = v.Snapdate.Format("2006-01-02 15:04:05")
		item.Sn = v.Sn
		item.Type = "出"
		if v.Inout == 1 {
			item.Type = "进"
		}
		if v.Carmodel == 0 {
			item.VehicleType = "运渣车"
		} else if v.Carmodel == 1 {
			item.VehicleType = "罐车"
		} else {
			item.VehicleType = "其他"
		}
		reply.List[i] = item
	}
	return reply, nil
}
