// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"encoding/json"
	"smart-site-management/consts"
	"smart-site-management/errors"
	"time"

	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"

	"github.com/jinzhu/gorm"
)

type TAlarmContact struct {
	Id        uint64 `gorm:"column:ID;primary_key"`
	ProjectId int64  `gorm:"column:ProjectId"`
	Phone     string `gorm:"column:Phone"`
	Email     string `gorm:"column:Email"`
	CreatedAt string `gorm:"column:CreatedAt"`
	UpdatedAt string `gorm:"column:UpdatedAt"`
}

func (TAlarmContact) TableName() string {
	return "AlarmContact"
}

func (p *TAlarmContact) Insert(db *gorm.DB) error {
	timeNow := time.Now().Format(consts.TimeSecondLayOut)
	p.CreatedAt = timeNow
	p.UpdatedAt = timeNow
	err := db.Table(p.TableName()).Create(p).Error
	if err != nil {
		fields, _ := json.Marshal(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into t_alarm_contact"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TAlarmContact) Delete(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Delete(p).Error
	if err != nil {
		fields, _ := json.Marshal(filter)
		logger.Error("mysql",
			zap.String("sql", "delete from t_alarm_contact"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
	}
	return err
}

// 通过结构体变量更新字段值, gorm库会忽略零值字段。就是字段值等于0, nil, "", false这些值会被忽略掉，不会更新。如果想更新零值，可以使用map类型替代结构体。
func (p *TAlarmContact) UpdateSome(db *gorm.DB, filed map[string]interface{}) error {
	if filed == nil {
		return errors.ParamsError
	}
	timeNow := time.Now().Format(consts.TimeSecondLayOut)
	filed["UpdatedAt"] = timeNow
	err := db.Model(p).Updates(filed).Error
	if err != nil {
		fields, _ := json.Marshal(filed)
		logger.Error("mysql",
			zap.String("sql", "update t_alarm_contact"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TAlarmContact) Query(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Find(p).Error
	if err != nil {
		fields, _ := json.Marshal(filter)
		logger.Error("mysql",
			zap.String("sql", "select from t_alarm_contact"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TAlarmContact) Find(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) error {
	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return err
	}
	err =  db.Table(p.TableName()).Where(cond, val...).First(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select from t_device "),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return nil
		}
		return  errors.DataBaseError
	}
	return err
}

func (p *TAlarmContact) QueryAll(db *gorm.DB, filter map[string]interface{}) (list []TAlarmContact, err error) {
	err = db.Where(filter).Find(&list).Error
	if err != nil {
		fields, _ := json.Marshal(filter)
		logger.Error("mysql",
			zap.String("sql", "select from t_alarm_contact"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
	}
	return list, err
}
