package model

import (
	"fmt"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"time"
	"smart-site-management/consts"
	"smart-site-management/errors"
	"github.com/jaryhe/gopkgs/logger"
)

type DeviceAll struct {
	ID int64 `gorm:"column:ID" json:"id"`
	SN string `gorm:"column:SN" json:"sn"`
	DeviceState int64 `gorm:"column:DeviceState" json:"devicestate"`
	Longitude string `gorm:"column:Longitude" json:"longitude"`
	Latitude string `gorm:"column:Latitude" json:"latitude"`
	DeviceCode uint32 `gorm:"column:DeviceCode"`
	DeviceName string `gorm:"column:DeviceName"`
	SubId int64 `gorm:"column:SubId"`
	Key string `gorm:"column:Key"`
	ProjectId int64 `gorm:"column:ProjectId"`
	ProviderId int64 `gorm:"column:ProviderId"`
	CreatedAt time.Time `gorm:"column:CreatedAt"`
	VerifyTime time.Time `gorm:"column:VerifyTime"`
	VerifyStatus int `gorm:"column:VerifyStatus"`
	ChannelId int64 `gorm:"column:ChannelId"`
}


func (DeviceAll) TableName() string {
	return "db_smart_v2.DeviceAll"
}

func (p *DeviceAll) Insert(db *gorm.DB) error {
	return db.Table(p.TableName()).Create(p).Error
}
/*
func (p *Dustdeviceinfo) Statistic(db *gorm.DB, projectId int64, deviceType int32, result interface{}) error {

	sql := fmt.Sprintf("select device_code as device_type, sum(status=0) as offline, sum(status=1) as online from t_device where project_id=%d group by type", projectId)
	if deviceType > 0 {
		sql = fmt.Sprintf("select device_code as device_type, sum(status=0) as offline, sum(status=1) as online from t_device where project_id=%d and type = %d group by type", projectId, deviceType)
	}
	err := db.Raw(sql).Scan(result).Error
	return err
}
*/
func (p *DeviceAll) Del(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
}

func (p *DeviceAll) Find(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).First(p).Error
}

func (p *DeviceAll) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	return db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
}

func (p *DeviceAll) FindSort(db *gorm.DB, where map[string]interface{}, sort string) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	ps := []DeviceAll{}
	err = db.Table(p.TableName()).Where(cond, val...).Order(sort).Limit(1).Find(&ps).Error
	if err != nil {
		return err
	}
	if len(ps) > 0 {
		*p = ps[0]
	}
	return nil
}

func (p *DeviceAll) Save(db *gorm.DB) error {
	return db.Save(p).Error
}

func (p *DeviceAll) Count(db *gorm.DB, where map[string]interface{}) (int64, error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error

		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	return ret, err
}

func (p *DeviceAll) List(db *gorm.DB, where map[string]interface{}, page int32, pageSize int32) (list []DeviceAll, err error) {
	offset := int64(page - 1)*int64(pageSize)
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return list, err
		}

		result := db.Table(p.TableName()).Where(cond, val...).Limit(pageSize).Offset(offset).Find(&list)

		return list, result.Error
	}
	result := db.Table(p.TableName()).Limit(pageSize).Offset(offset).Find(&list)

	return list, result.Error

}


type DeviceAllItem struct {
	Id                   int64
	Sn                   string
	TypeCode             int32
	TypeName             string
	ProjectName          string
	ProjectId            int64
	SafetyRecordNo       string
	CreatedTime          time.Time
	Status               int32
	State int32
	ProjectApproveTime time.Time
	Key string
	ProviderName string
	SocialCode string
	Name string
	Ip string
	Port int
	MediaTransport string
	ChannelCount int

}

func deviceAllListSql(req DeviceAllListRequest)(string, string, []interface{}) {
	sql := fmt.Sprintf("select t1.DeviceCode as type_code, t1.ID as id, t1.DeviceName as name, t1.SN as sn, t1.VerifyTime as project_approve_time, "+
		"t1.CreatedAt as created_time, t1.DeviceState as state, t1.VerifyStatus as status, t2.SafetyNo as safety_record_no, t2.Name as project_name, t3.Name as provider_name, t3.SocialCode as social_code from DeviceAll as t1 left join ProjectInfo as t2 on t1.ProjectId=t2.ID left join Provider as t3 on t1.ProviderId=t3.ID")
	countSql := fmt.Sprintf("select count(1) as count from DeviceAll as t1 left join ProjectInfo as t2 on t1.ProjectId=t2.ID ")
	args := []interface{}{}
	whereArray := []string{}
	if req.ProviderId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.ProviderId=%d", req.ProviderId))
	}

	if req.ProjectId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.ProjectId=%d", req.ProjectId))
	}
	if req.State >= 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.DeviceState=%d", req.State))
	}

	if req.Cid > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t2.Cid=%d", req.Cid))
	}

	if req.Filter != "" {
		whereArray = append(whereArray, fmt.Sprintf("(t1.SN like '%%%s%%' or t1.DeviceName like '%%%s%%')",  req.Filter, req.Filter))
	}

	if req.CanDel {
		req.IsAll = true
		req.StatusFilter = []int32{consts.DeviceStatusAddAuditted}
		whereArray = append(whereArray, fmt.Sprintf("(select count(1) from DeviceDelJob where DeviceId=t1.ID and (Status = 0 or Status = 1)) = 0"))
	}
	if len(req.StatusFilter) > 0 {
		args = append(args, req.StatusFilter)
		whereArray = append(whereArray, fmt.Sprintf("t1.VerifyStatus in(?)"))
	}
	where := ""

	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf(" where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}
	offset := (req.Page - 1) *int32(PageSize)
	if req.IsAll {
		sql = fmt.Sprintf("%s %s order by t1.CreatedAt desc", sql, where)
		countSql = fmt.Sprintf("%s %s", countSql, where)
	} else {
		sql = fmt.Sprintf("%s %s order by t1.CreatedAt desc limit %d offset %d", sql, where, PageSize, offset)
		countSql = fmt.Sprintf("%s %s", countSql, where)
	}
	return sql, countSql, args
}

type DeviceAllListRequest struct {
	ProviderId int64
	Filter string
	StatusFilter []int32
	IsAll bool
	CanDel bool
	Page int32
	ProjectId int64
	Cid int64
	State int32
}

func (p *DeviceAll) DeviceAllList(db *gorm.DB, req DeviceAllListRequest) ([]DeviceAllItem, int64, error) {
	type ResultCount struct {
		Count int64
	}
	array := []ResultCount{}
	ret := []DeviceAllItem{}
	var err error

	sql, countSql, args := deviceAllListSql(req)

	err = db.Raw(countSql, args...).Scan(&array).Error
	if err != nil {
		return nil, 0, err
	}
	if len(array) == 0 {
		return nil, 0, nil
	}
	if array[0].Count == 0 {
		return nil, 0, nil
	}

	err = db.Raw(sql, args...).Scan(&ret).Error
	if err != nil {
		return nil, array[0].Count, err
	}
	return ret, array[0].Count, nil
}


// 按设备类型统计
func (p *DeviceAll) Statistic(db *gorm.DB, projectId int64, deviceType int32, result interface{}) error {

	sql := fmt.Sprintf("select DeviceCode as device_type, sum(DeviceState=0) as offline, sum(DeviceState=1) as online from DeviceAll where ProjectId=%d group by DeviceCode", projectId)
	if deviceType > 0 {
		sql = fmt.Sprintf("select DeviceCode as device_type, sum(DeviceState=0) as offline, sum(DeviceState=1) as online from DeviceAll where ProjectId=%d and DeviceCode = %d group by DeviceCode", projectId, deviceType)
	}
	err := db.Raw(sql).Scan(result).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

type DeviceStatisticsItem struct {
	TypeCode int32
	Total int32
	Online int32
}


func (p *DeviceAll) StatisticWithProject(db *gorm.DB, projectId int64) ([]DeviceStatisticsItem, error) {
	sql := fmt.Sprintf("select DeviceCode as type_code, sum(1) as total, sum(DeviceState=1) as online from DeviceAll where ProjectId=? and VerifyStatus=1 and DeviceCode <> %d group by DeviceCode", DeviceTypeVedio)
	array := []DeviceStatisticsItem{}
	err := db.Raw(sql, projectId).Scan(&array).Error

	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("fields", fmt.Sprintf("%d", projectId)),
			zap.String("error", err.Error()))
		return nil,  errors.DataBaseError
	}

	vedioArray := []DeviceStatisticsItem{}
	sql = fmt.Sprintf("select t3.DeviceCode as type_code, sum(1) as total, sum(t1.Alive=1) as online from VSSChannelTbl as t1 left join VSSDevTbl as t2 on t1.DevPubID=t2.DevPubID left join DeviceAll as t3 on t3.SN=t1.DevPubID where t3.DeviceCode=%d and t3.VerifyStatus=1 and t3.ProjectId=? group by t3.DeviceCode", DeviceTypeVedio)
	err = db.Raw(sql, projectId).Scan(&vedioArray).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("fields", fmt.Sprintf("%d", projectId)),
			zap.String("error", err.Error()))
		return nil,  errors.DataBaseError
	}
	if len(vedioArray) > 0 {
		return append(array, vedioArray...), nil
	}

	return array, nil
}

type IotType struct {
	Code int32
	Name string
	Id int64
}
func IotTypeList(db *gorm.DB, projectId int64, codes []int32) ([]IotType, error) {
	result := []IotType{}
	sql := fmt.Sprintf("select t1.DeviceCode as code, t2.DeviceType as name, t2.ID from DeviceAll as t1 left join DeviceTypeItem as t2 on t1.DeviceCode=t2.DeviceCode where t1.ProjectId = %d and t1.VerifyStatus = 1 and t1.DeviceCode in(?) group by t1.DeviceCode", projectId)
	err := db.Raw(sql, codes).Scan(&result).Error
	if err != nil {
		return nil, errors.DataBaseError
	}
	return result, nil
}
