package model

import (
	"fmt"
	"github.com/jinzhu/gorm"
)

type DustMonitorInfo struct {
	ID int64 `gorm:"column:ID"`
	Code string `gorm:"column:Code" json:"Code"`
	Name string `gorm:"column:Name" json:"Name"`
	DeviceID string `gorm:"column:DeviceID" json:"DeviceID"`
	MonitoringTime string `gorm:"column:MonitoringTime" json:"MonitoringTime"`
	SaveTime string `gorm:"column:SaveTime" json:"SaveTime"`
	Pm10Value float64 `gorm:"column:Pm10Value" json:"Pm10Value"`
	Pm25Value float64 `gorm:"column:Pm25Value" json:"Pm25Value"`
	Voice float64 `gorm:"column:Voice" json:"Voice"`
	Temperature float64 `gorm:"column:Temperature" json:"Temperature"`
	Humidity float64 `gorm:"column:Humidity" json:"Humidity"`
	WindSpeed float64 `gorm:"column:WindSpeed" json:"WindSpeed"`
	WindDirection float64 `gorm:"column:WindDirection" json:"WindDirection"`
	Atmospheric float64 `gorm:"column:Atmospheric" json:"Atmospheric"`
	Type uint32 `gorm:"column:Type" json:"Type"`
	Pm25Monitor float64 `gorm:"column:Pm25Monitor" json:"Pm25Monitor"`
	Pm10Monitor float64 `gorm:"column:Pm10Monitor" json:"Pm10Monitor"`
	SprayStatus int64 `gorm:"column:SprayStatus" json:"SprayStatus"`
	ProjectId int64 `gorm:"column:ProjectId" json:"ProjectId"`
	SN string `gorm:"column:SN" json:"SN"`
	Rain float64 `gorm:"column:Rain" json:"Rain"`
	Tsp float64 `gorm:"column:Tsp" json:"Tsp"`
	// 仅供influxdb使用
	Time string `gorm:"-" json:"time"`
}

func (DustMonitorInfo) TableName() string {
	return "DustMonitorInfo"
}

func (p *DustMonitorInfo) Insert(db *gorm.DB) error {
	return db.Table(p.TableName()).Create(p).Error
}

func (p *DustMonitorInfo) Find(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) error {
	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).First(p).Error
}

func (p *DustMonitorInfo) FindSort(db *gorm.DB, where map[string]interface{}, sort string) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	ps := []DustMonitorInfo{}
	err = db.Table(p.TableName()).Where(cond, val...).Order(sort).Limit(1).Find(&ps).Error
	if err != nil {
		return err
	}
	if len(ps) > 0 {
		*p = ps[0]
	}
	return nil
}


func (p *DustMonitorInfo) All(db *gorm.DB, where map[string]interface{}) (list []DustMonitorInfo, err error) {

	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return nil, err
		}
		result := db.Table(p.TableName()).Where(cond, val).Find(&list)
		return list, result.Error
	}
	result := db.Table(p.TableName()).Find(&list)

	return list, result.Error
}

func (p *DustMonitorInfo) Count(db *gorm.DB, where map[string]interface{}) (int64, error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error

		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	return ret, err
}

func (p *DustMonitorInfo) Del(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
}

func GetPm25Avg(db *gorm.DB, Sn string, start string, end string) (float64, error) {
	type Result struct {
		Pm25 float64
	}
	array := []Result{}
	sql := fmt.Sprintf("select AVG(Pm25Monitor) as pm25 from DustMonitorInfo where time >= '%s' and time < '%s' and SN='%s'", start, end, Sn)
	err := db.Raw(sql).Scan(&array).Error
	if err != nil {
		return 0, err
	}
	if len(array) == 0 {
		return 0, nil
	}
	return array[0].Pm25, nil

}

