// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"go.uber.org/zap"
	"smart-site-management/errors"
	"time"
	"fmt"
	"github.com/jinzhu/gorm"
	"github.com/jaryhe/gopkgs/logger"
	"encoding/json"
	"smart-site-management/consts"
)

const (
	DeviceTypeVedio = 9
	DeviceTypeDust = 5
	DeviceTypeTower = 1
	DeviceTypeHighFramework = 7
	DeviceTypeUnloading = 6
	DeviceTypeWireless = 8
	DeviceTypeSlag = 2
	DeviceTypeAttendance = 3
	DeviceTypeLift = 4

)

type DustDeviceInfo struct {
	ID int64 `gorm:"column:ID;PRIMARY_KEY" json:"ID"`
	Code string `gorm:"column:Code" json:"code"`
	Name string `gorm:"column:Name" json:"name"`
	DeviceId string `gorm:"column:DeviceId" json:"deviceid"`
	DeviceModel string `gorm:"column:DeviceModel" json:"devicemodel"`
	DeviceName string `gorm:"column:DeviceName" json:"devicename"`
	Manufacturer string `gorm:"column:Manufacturer" json:"manufacturer"`
	Batch string `gorm:"column:Batch" json:"batch"`
	SN string `gorm:"column:SN" json:"sn"`
	SIM string `gorm:"column:SIM" json:"sim"`
	DeviceState int64 `gorm:"column:DeviceState" json:"devicestate"`
	Unit string `gorm:"column:Unit" json:"unit"`
	Person string `gorm:"column:Person" json:"person"`
	Phone string `gorm:"column:phone" json:"phone"`
	Remark string `gorm:"column:remark" json:"remark"`
	Longitude string `gorm:"column:Longitude" json:"longitude"`
	Latitude string `gorm:"column:Latitude" json:"latitude"`
	Url string `gorm:"column:Url" json:"url"`
	VerifyTime time.Time `gorm:"column:VerifyTime"`
	VerifyStatus int `gorm:"column:VerifyStatus"`
	Key string `gorm:"column:Key"`
	ProjectId int64 `gorm:"column:ProjectId"`
	ProviderId int64 `gorm:"column:ProviderId"`
	CreatedAt time.Time `gorm:"column:CreatedAt"`
}

func (DustDeviceInfo) TableName() string {
	return "DustDeviceInfo"
}

func (p *DustDeviceInfo) Insert(db *gorm.DB) error {
	err := db.Table(p.TableName()).Create(p).Error
	if err != nil {
		fields, _ := json.Marshal(p)
		logger.Error("mysql",
			zap.String("sql", "insert info t_device "),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *DustDeviceInfo) List(db *gorm.DB, where map[string]interface{}, page int32, pageSize int32) (list []DustDeviceInfo, err error) {
	offset := (page - 1) *pageSize
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return list, err
		}

		err = db.Table(p.TableName()).Where(cond, val...).Limit(pageSize).Offset(offset).Find(&list).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select * from t_device"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			if err == gorm.ErrRecordNotFound {
				return list, nil
			}
			return  list, errors.DataBaseError
		}

		return list, err
	}
	err = db.Table(p.TableName()).Limit(pageSize).Offset(offset).Find(&list).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select * from t_device"),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return list, nil
		}
		return  list, errors.DataBaseError
	}

	return list, err

}



func (p *DustDeviceInfo) Del(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err = db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "delete from t_device "),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *DustDeviceInfo) Find(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err =  db.Table(p.TableName()).Where(cond, val...).First(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select from t_device "),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return nil
		}
		return  errors.DataBaseError
	}
	return err
}

func (p *DustDeviceInfo) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	err = db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		updateValues, _ := json.Marshal(values)
		logger.Error("mysql",
			zap.String("sql", "update t_device "),
			zap.String("fields", string(fields)),
			zap.String("values", string(updateValues)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *DustDeviceInfo) Count(db *gorm.DB, where map[string]interface{}) (int64, error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select count(1) from t_device "),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			return  ret, errors.DataBaseError
		}
		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select count(1) from t_device "),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  ret, errors.DataBaseError
	}
	return ret, err
}

type DustDeviceItem struct {
	Id                   int64
	Sn                   string
	TypeCode             int32
	TypeName             string
	ProjectName          string
	ProjectId            int64
	SafetyRecordNo       string
	CreatedTime          time.Time
	Status               int32
	State int32
	VerifyTime time.Time
	Key string
	ProviderName string
	SocialCode string
	Name string
	Manufacturer string
	Url string
	Batch string
	Person string
	Phone string
	Unit string
	Lon string
	Lat string
	Model string
	ChannelId int64
}


func dustDeviceListSql(req DustDeviceListRequest)(string, string, []interface{}) {
	sql := fmt.Sprintf("select t1.ID as id, t1.ChannelId as channel_id, t1.DeviceModel as model, t1.Batch as batch, t1.Person as person, t1.Phone as phone, t1.Manufacturer as manufacturer, t1.Longitude as lon, t1.Latitude as lat, t1.Url as url,  t1.Key as 'key', t1.DeviceName as name, t1.SN as sn, t1.VerifyTime as verify_time, "+
		"t1.CreatedAt as created_time, t1.DeviceState as state, t1.VerifyStatus as status, t2.SafetyNo as safety_record_no, t2.Name as project_name, t3.Name as provider_name, t3.SocialCode as social_code from DustDeviceInfo as t1 left join ProjectInfo as t2 on t1.ProjectId=t2.ID left join Provider as t3 on t1.ProviderId=t3.ID")
	countSql := fmt.Sprintf("select count(1) as count from DustDeviceInfo as t1 left join ProjectInfo as t2 on t1.ProjectId=t2.ID ")
	args := []interface{}{}
	whereArray := []string{}
	if req.ProjectId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.ProjectId=%d", req.ProjectId))
	}
	if req.Cid > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t2.Cid=%d", req.Cid))
	}

	if req.Filter != "" {
		whereArray = append(whereArray, fmt.Sprintf("(t1.SN like '%%%s%%' or t1.DeviceName like '%%%s%%')", req.Filter, req.Filter))
	}

	if req.State >= 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.DeviceState=%d", req.State))
	}

	if req.CanDel {
		req.IsAll = true
		req.StatusFilter = []int32{consts.DeviceStatusAddAuditted}
		whereArray = append(whereArray, fmt.Sprintf("(select count(1) from DeviceDelJob where DeviceId=t1.ID and (Status = 0 or Status = 1)) = 0"))
	}
	if len(req.StatusFilter) > 0 {
		args = append(args, req.StatusFilter)
		whereArray = append(whereArray, fmt.Sprintf("t1.VerifyStatus in(?)"))
	}
	where := ""

	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf(" where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}
	offset := (req.Page - 1) *int32(PageSize)
	if req.IsAll {
		sql = fmt.Sprintf("%s %s order by t1.CreatedAt desc", sql, where)
		countSql = fmt.Sprintf("%s %s", countSql, where)
	} else {
		sql = fmt.Sprintf("%s %s order by t1.CreatedAt desc limit %d offset %d", sql, where, PageSize, offset)
		countSql = fmt.Sprintf("%s %s", countSql, where)
	}
	return sql, countSql, args
}

type DustDeviceListRequest struct {
	ProviderId int64
	Filter string
	StatusFilter []int32
	IsAll bool
	CanDel bool
	Page int32
	ProjectId int64
	Cid int64
	State int32
}

func (p *DustDeviceInfo) DustDeviceList(db *gorm.DB, req DustDeviceListRequest) ([]DustDeviceItem, int64, error) {
	type ResultCount struct {
		Count int64
	}
	array := []ResultCount{}
	ret := []DustDeviceItem{}
	var err error

	sql, countSql, args := dustDeviceListSql(req)

	err = db.Raw(countSql, args...).Scan(&array).Error
	if err != nil {
		return nil, 0, err
	}
	if len(array) == 0 {
		return nil, 0, nil
	}
	if array[0].Count == 0 {
		return nil, 0, nil
	}

	err = db.Raw(sql, args...).Scan(&ret).Error
	if err != nil {
		return nil, array[0].Count, err
	}
	return ret, array[0].Count, nil
}




