// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package model

import (
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"time"
	"github.com/jaryhe/gopkgs/logger"
	"encoding/json"
)

const (
	// 未完工
	ProjectProgressUnfinish = iota
	// 已完工
	ProjectProgressFinish
	// 进行中
	ProjectProgressWorking
)

type TProjectSchedule struct {
	ID int64 `gorm:"column:ID"`
	ProjectId int64 `gorm:"column:ProjectId"`
	Name string `gorm:"column:Name"`
	ExpectStartTime time.Time `gorm:"column:ExpectStartTime"`
	ExpectEndTime time.Time `gorm:"column:ExpectEndTime"`
	RealityEndTime time.Time `gorm:"column:RealityEndTime"`
	RealityStartTime time.Time `gorm:"column:RealityStartTime"`
	Image string `gorm:"column:Image"`
	Status int64 `gorm:"column:Status"`
	CreatedAt time.Time `gorm:"column:CreatedAt"`
	UpdatedAt time.Time `gorm:"column:UpdatedAt"`
}


func (TProjectSchedule) TableName() string {
	return "ProjectSchedule"
}

func (p *TProjectSchedule) Insert(db *gorm.DB) error {
	err :=  db.Create(p).Error
	if err != nil {
		fields, _ := json.Marshal(p)
		logger.Error("mysql",
			zap.String("sql", "insert into t_project_schedule"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *TProjectSchedule) Find(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) error {
	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return err
	}
	err = db.Table(p.TableName()).Where(cond, val...).First(p).Error
	if err != nil {
		fields, _ := json.Marshal(val)
		logger.Error("mysql",
			zap.String("sql", "insert into t_project_schedule"),
			zap.String("fields", cond + ","+string(fields)),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return nil
		}
		return  errors.DataBaseError
	}
	return err
}

func (p *TProjectSchedule) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	err = db.Table(p.TableName()).Table(p.TableName()).Where(cond, val...).Updates(values).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		updateValues, _ := json.Marshal(values)
		logger.Error("mysql",
			zap.String("sql", "update t_project_schedule"),
			zap.String("fields", string(fields)),
			zap.String("values", string(updateValues)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}


func (p *TProjectSchedule) List(db *gorm.DB, where map[string]interface{}, page int) (list []TProjectSchedule, err error) {
	offset := (page - 1) * PageSize
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return list, err
		}

		err = db.Table(p.TableName()).Where(cond, val...).Limit(PageSize).Offset(offset).Find(&list).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select * from t_project_schedule"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			if err == gorm.ErrRecordNotFound {
				return list, nil
			}
			return  list, errors.DataBaseError
		}
		return list, err
	}
	err = db.Table(p.TableName()).Limit(10).Offset(offset).Find(&list).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select * from t_project_schedule limit 10"),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return list, nil
		}
		return  list, errors.DataBaseError
	}
	return list, err
}

func (p *TProjectSchedule) ListNoPage(db *gorm.DB, where map[string]interface{}) (list []TProjectSchedule, err error) {

	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return list, err
		}

		err = db.Table(p.TableName()).Where(cond, val...).Order("ExpectStartTime").Find(&list).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select * from t_project_schedule"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			if err == gorm.ErrRecordNotFound {
				return list, nil
			}
			return  list, errors.DataBaseError
		}
		return list, err
	}
	err = db.Table(p.TableName()).Find(&list).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select * from t_project_schedule"),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return list, nil
		}
		return  list, errors.DataBaseError
	}
	return list, err

}

func (p *TProjectSchedule) Count(db *gorm.DB, where map[string]interface{}) (int64, error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select count(1) from t_project_schedule"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))

			return  ret, errors.DataBaseError
		}
		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	if err != nil {

		logger.Error("mysql",
			zap.String("sql", "select count(1) from t_project_schedule"),
			zap.String("error", err.Error()))

		return  ret, errors.DataBaseError
	}
	return ret, err
}

func (p *TProjectSchedule) Del(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err = db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "delete from t_project_schedule"),
			zap.String("fields",string(fields)),
			zap.String("error", err.Error()))

		return   errors.DataBaseError
	}
	return err
}

/* 废弃
func ProgressDelay(db *gorm.DB) error {
	now := time.Now().Format("2006-01-02")
	sql := fmt.Sprintf("update t_project_schedule set status=%d where status = %d and expect_end_time < '%s'", ProjectProgressDelay, ProjectProgressUnfinish, now)
	err := db.Exec(sql).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("error", err.Error()))

		return   errors.DataBaseError
	}
	return err
}
 */
