// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package model

import (
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"time"
	"github.com/jaryhe/gopkgs/logger"
	"encoding/json"
)

type TSiteUser struct {
	Id int64 `gorm:"column:ID" json:"id" form:"id"`
	Username string `gorm:"column:UserName" json:"username" form:"username"`
	Passwd string `gorm:"column:Passwd" json:"passwd" form:"passwd"`
	SiteUser int64 `gorm:"column:Site_user" json:"site_user" form:"site_user"`
	CreatedAt time.Time `gorm:"column:CreatedAt" json:"created_at" form:"created_at"`
	UpdatedAt time.Time `gorm:"column:UpdatedAt" json:"updated_at" form:"updated_at"`
	UserType int64 `gorm:"column:UserType" json:"user_type" form:"user_type"`
	Enable bool `gorm:"column:Enable"`
	ProjectId int64 `gorm:"column:ProjectId"`
	Phone string `gorm:"column:Phone"`
	Email string `gorm:"column:email"`
}


func (TSiteUser) TableName() string {
	return "ProjectUser"
}


func (p *TSiteUser) Insert(db *gorm.DB) error {
	err := db.Table(p.TableName()).Create(p).Error
	if err != nil {
		fields, _ := json.Marshal(p)
		logger.Error("mysql",
			zap.String("sql", "insert into t_site_user"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *TSiteUser) Find(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) error {
	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return err
	}
	err =  db.Table(p.TableName()).Where(cond, val...).First(p).Error
	if err != nil {
		fields, _ := json.Marshal(val)
		logger.Error("mysql",
			zap.String("sql", "select from t_site_user"),
			zap.String("fields", cond +","+string(fields)),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return nil
		}
		return  errors.DataBaseError
	}
	return err
}

func (p *TSiteUser) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err = db.Table(p.TableName()).Table(p.TableName()).Where(cond, val...).Updates(values).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		updateValues, _ := json.Marshal(values)
		logger.Error("mysql",
			zap.String("sql", "update t_site_user"),
			zap.String("fields",string(fields)),
			zap.String("values",string(updateValues)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}
