// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"fmt"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"time"
)

type TStaffAttendance struct {
	ID         int64  `gorm:"column:ID;primary_key"`
	Code string `gorm:"column:Code" json:"code"`
	TeamSysNo int `gorm:"column:TeamSysNo" json:"team_sys_no"`
	TeamName string `gorm:"column:TeamName" json:"team_name"`
	WorkName string `gorm:"column:WorkName" json:"work_name"`
	IDCardType int `gorm:"column:IDCardType" json:"id_card_type"`
	IDCardNumber string `gorm:"column:IDCardNumber" json:"id_card_number"`
	Date time.Time `gorm:"column:Date" json:"date"`
	Direction int `gorm:"column:Direction" json:"direction"`
	Temperature float64 `gorm:"column:Temperature" json:"temperature"`
	ImageURL string `gorm:"column:ImageURL" json:"image_url"`
	AttendType int `gorm:"column:AttendType" json:"attend_type"`
	Lng float64 `gorm:"column:Lng" json:"lng"`
	Lat float64 `gorm:"column:Lat" json:"lat"`
	ProjectId  int64  `gorm:"column:ProjectId" json:"project_id"`
	WorkNo     string `gorm:"column:WorkNo" json:"work_no"`
	Sn string `gorm:"column:Sn" json:"sn"`
	TabType int `gorm:"-"` // 用于判断表0 t_staff_attendance, 1-12 t_staff_attendance_monthx, 13 t_staff_attendance_day
}

const (
	AttendanceTabDay = iota
	AttendanceTabMonth1
	AttendanceTabMonth2
	AttendanceTabMonth3
	AttendanceTabMonth4
	AttendanceTabMonth5
	AttendanceTabMonth6
	AttendanceTabMonth7
	AttendanceTabMonth8
	AttendanceTabMonth9
	AttendanceTabMonth10
	AttendanceTabMonth11
	AttendanceTabMonth12
)

func (p TStaffAttendance) TableName() string {

	if p.TabType >=AttendanceTabMonth1 && p.TabType <= AttendanceTabMonth12 {
		return fmt.Sprintf("db_smart_attendance.WorkerAttendanceInfoMonth%d", p.TabType)
	}
	return "db_smart_attendance.WorkerAttendanceInfoDay"
}


type TStaffAttendanceLast struct {
	WorkNo string
	Name string
	Photo string
	RecogTime time.Time
	InOut int
	TeamName string
	WorkType string
}

func (p *TStaffAttendance) Last(db *gorm.DB, projectId int64) (TStaffAttendanceLast, error) {
	result := TStaffAttendanceLast{}
	sql := fmt.Sprintf("select t1.WorkNo as work_no, t1.Direction as in_out, t1.Date as recog_time, t2.WorkName as name, t2.HeadImage as photo, t2.TeamName as team_name, t2.WorkType as work_type from db_smart_attendance.WorkerAttendanceInfoDay as t1 left join db_smart_attendance.ProjectWorkInfo as t2 on t1.WorkNo=t2.WorkNo")
	whereArray := []string{}
	if projectId > 0 {
		whereArray = append(whereArray,fmt.Sprintf("t1.ProjectId=%d", projectId))
	}
	where := ""
	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf(" where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}
	sql = fmt.Sprintf("%s %s order by t1.Date desc limit 1", sql, where)

	ret := []TStaffAttendanceLast{}
	err := db.Raw(sql).Scan(&ret).Error

	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("error", err.Error()))
		return  result, errors.DataBaseError
	}

	if len(ret) == 0 {
		return result, nil
	}
	return ret[0], nil

}


type WorkTypeStaticItem struct {
	TypeCode string
	TypeName string
	StaffCount int
	StaffAttendanceCount int
	Id int64
}

func staffWorkTypeSql(projectId int64)(string) {
	sql := fmt.Sprintf("select t1.Code as type_code, t1.ID as id, t1.Type as type_name, count(distinct t2.WorkNo) as staff_count, count(distinct t3.WorkNo) as staff_attendance_count from db_smart_attendance.WorkType as t1 left join db_smart_attendance.ProjectWorkInfo as t2 on t1.Code=t2.WorkType and t2.ProjectId=%d left join db_smart_attendance.WorkerAttendanceInfoDay as t3 on t3.WorkNo=t2.WorkNo and Date(t3.Date)='%s' and t3.ProjectId=%d", projectId, time.Now().Format("2006-01-02"), projectId)
	whereArray := []string{}
	if projectId > 0 {
		//whereArray = append(whereArray, fmt.Sprintf("t2.project_id=%d", projectId))
	}
	where := ""

	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf(" where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}


	sql = fmt.Sprintf("%s %s group by t1.Code", sql, where)

	return sql
}

func StaffWorkTypeStaticList(db *gorm.DB, projectId int64) ([]WorkTypeStaticItem, error) {
	ret := []WorkTypeStaticItem{}
	var err error

	sql := staffWorkTypeSql(projectId)

	err = db.Raw(sql).Scan(&ret).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("error", err.Error()))
		return  nil, errors.DataBaseError
	}

	return ret,  nil
}



type StaffTypeStaticItem struct {
	TypeCode int32
	TypeName string
	StaffCount int
	StaffAttendanceCount int
	StaffInCount int
}

type StaffTypeCount struct {
	TypeCode int32
	TypeName string
	Count int
}


func StaffTypeStaticList(db *gorm.DB, projectId int64) ([]StaffTypeStaticItem, error) {

	var err error

	totalCountArray := []StaffTypeCount{}
	attendanceCountArray := []StaffTypeCount{}
	inCountArray := []StaffTypeCount{}
	sqlTotalCount := fmt.Sprintf("select a.Code as type_code, a.Type as type_name, count(distinct b.work_no) as count from db_smart_attendance.ManagerType as a left join ( select t2.WorkNo as work_no, t3.Ptype as ptype from db_smart_attendance.ProjectWorkInfo as t2 left join db_smart_attendance.ProjectPMInfo as t3 on t2.IDCardNumber=t3.PMIDCardNumber and t2.ProjectId=t3.ProjectId where t2.ProjectId=%d) as b on a.Code=b.ptype group by a.Code order by a.Code ", projectId)
	sqlAttendanceCount := fmt.Sprintf("select a.Code as type_code, a.Type as type_name, count(distinct b.work_no) as count from db_smart_attendance.ManagerType as a left join ( select t2.WorkNo as work_no, t3.Ptype as ptype from db_smart_attendance.WorkerAttendanceInfoDay as t2 left join db_smart_attendance.ProjectPMInfo as t3 on t2.IDCardNumber=t3.PMIDCardNumber and t2.ProjectId=t3.ProjectId where t2.ProjectId=%d and Date(t2.Date)='%s') as b on a.Code=b.ptype group by a.Code order by a.Code", projectId, time.Now().Format("2006-01-02"))
	sqlInCount := fmt.Sprintf("select a.Code as type_code, a.Type as type_name, count(distinct c.work_no) as count from db_smart_attendance.ManagerType as a left join ( select * from(select t2.WorkNo as work_no, t2.Direction as direction, t3.Ptype as ptype from db_smart_attendance.WorkerAttendanceInfoDay as t2 left join db_smart_attendance.ProjectPMInfo as t3 on t2.IDCardNumber=t3.PMIDCardNumber and t2.ProjectId=t3.ProjectId where t2.ProjectId=%d and Date(t2.Date)='%s' order by t2.Date desc) as b group by b.work_no having b.direction=1) as c on a.Code=c.ptype group by a.Code order by a.Code", projectId, time.Now().Format("2006-01-02"))

	err = db.Raw(sqlTotalCount).Scan(&totalCountArray).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sqlTotalCount),
			zap.String("error", err.Error()))
		return  nil, errors.DataBaseError
	}

	err = db.Raw(sqlAttendanceCount).Scan(&attendanceCountArray).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sqlAttendanceCount),
			zap.String("error", err.Error()))
		return  nil, errors.DataBaseError
	}

	err = db.Raw(sqlInCount).Scan(&inCountArray).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sqlInCount),
			zap.String("error", err.Error()))
		return  nil, errors.DataBaseError
	}
	if len(inCountArray) != len(totalCountArray) || len(totalCountArray) != len(attendanceCountArray) {
		return nil, errors.SystemError
	}
	ret := make([]StaffTypeStaticItem, len(inCountArray))
	for i, v := range totalCountArray {
		ret[i].TypeCode = v.TypeCode
		ret[i].TypeName = v.TypeName
		ret[i].StaffAttendanceCount = attendanceCountArray[i].Count
		ret[i].StaffCount = totalCountArray[i].Count
		ret[i].StaffInCount = inCountArray[i].Count
	}

	return ret,  nil
}


type StaffCompanyStaticItem struct {
	CompanyName string
	StaffCount int
	StaffAttendanceCount int
	LaborCount int
}

func staffCompanySql(projectId int64)(string) {
	sql := fmt.Sprintf("select t1.CorpName as company_name, count(distinct t1.WorkNo) as staff_count, count(distinct t2.WorkNo) as staff_attendance_count, count(distinct t3.WorkNo) as labor_count from db_smart_attendance.ProjectWorkInfo as t1 left join db_smart_attendance.WorkerAttendanceInfoDay as t2 on t1.WorkNo=t2.WorkNo and Date(t2.Date)='%s' left join db_smart_attendance.WorkerAttendanceInfoDay as t3 on t1.WorkNo=t3.WorkNo and Date(t3.Date)='%s' and t3.TeamSysNo != -100", time.Now().Format("2006-01-02"), time.Now().Format("2006-01-02"))
	whereArray := []string{}
	if projectId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.ProjectId=%d", projectId))
	}
	where := ""

	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf(" where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}


	sql = fmt.Sprintf("%s %s group by t1.CorpCode", sql, where)

	return sql
}

func StaffCompanyStaticList(db *gorm.DB, projectId int64) ([]StaffCompanyStaticItem, error) {
	ret := []StaffCompanyStaticItem{}
	var err error

	sql := staffCompanySql(projectId)

	err = db.Raw(sql).Scan(&ret).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("error", err.Error()))
		return  nil, errors.DataBaseError
	}

	return ret,  nil
}

type StaffAttendanceItem struct {
	Id int64
	Name string
	WorkNo string
	Gender string
	StaffType int
	WorkType string
	DayTime time.Time
	Direction int
	Attendance string
}

type StaffAttendanceRequest struct {
	Page int32
	PageSize int32
	Filter string
	ProjectId int64
	Start string
	End string
}

func (p *TStaffAttendance) List(db *gorm.DB, req StaffAttendanceRequest) (list []StaffAttendanceItem, total int64, err error) {
	offset := (req.Page - 1) *req.PageSize
	where := ""
	whereArray := []string{}
	type ResultCount struct {
		Count int64
	}
	sql := fmt.Sprintf("select t1.ID as id, group_concat(concat_ws('-',t1.Date,t1.Direction) order by t1.Date desc separator '/') as attendance, date(t1.Date) as day_time, t2.WorkName as name, t2.Gender as gender, t2.WorkNo as work_no, t2.WorkType as work_type, t3.PType as staff_type from %s as t1 left join db_smart_attendance.ProjectWorkInfo as t2 on t1.ProjectId= t2.ProjectId and t1.WorkNo=t2.WorkNo left join db_smart_attendance.ProjectPMInfo as t3 on t2.IDCardNumber=t3.PMIDCardNumber and t2.ProjectId=t3.ProjectId", p.TableName())
	countSql := fmt.Sprintf("select 1 as count from %s as t1 left join db_smart_attendance.ProjectWorkInfo as t2 on t1.ProjectId= t2.ProjectId and t1.WorkNo=t2.WorkNo left join db_smart_attendance.ProjectPMInfo as t3 on t2.IDCardNumber=t3.PMIDCardNumber and t2.ProjectId=t3.ProjectId", p.TableName())
	if req.ProjectId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.ProjectId=%d", req.ProjectId))
	}
	if req.Filter != "" {
		whereArray = append(whereArray, fmt.Sprintf("(t2.WorkName like '%%%s%%') ", req.Filter))
	}
	whereArray = append(whereArray, fmt.Sprintf("t2.WorkNo is not NULL"))
	if req.Start != "" {
		//sql = fmt.Sprintf("%s and t4.day_time >= '%s'", sql, req.Start)
		//countSql = fmt.Sprintf("%s and t4.day_time >= '%s'",countSql, req.Start)
		whereArray = append(whereArray, fmt.Sprintf("t1.Date >= '%s'", req.Start))
	}
	if req.End != "" {
		//sql = fmt.Sprintf("%s and t4.day_time < '%s'", sql, req.End)
		//countSql = fmt.Sprintf("%s and t4.day_time < '%s'",countSql, req.End)
		whereArray = append(whereArray, fmt.Sprintf("t1.Date < '%s'", req.End))
	}
	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf(" where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}
	sql = fmt.Sprintf("%s %s group by date(t1.Date), t1.WorkNo limit %d offset %d", sql, where, req.PageSize, offset)
	countSql = fmt.Sprintf("%s %s group by date(t1.Date), t1.WorkNo", countSql, where)
	countSql = fmt.Sprintf("select count(1) as count from (%s) as a", countSql)

	array := []ResultCount{}
	ret := []StaffAttendanceItem{}

	err = db.Raw(countSql).Scan(&array).Error

	if err != nil {
		logger.Error("mysql",
			zap.String("sql", countSql),
			zap.String("error", err.Error()))
		return  nil, 0, errors.DataBaseError
	}
	if len(array) == 0 {
		return nil, 0, nil
	}
	if array[0].Count == 0 {
		return nil, 0, nil
	}

	err = db.Raw(sql).Scan(&ret).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("error", err.Error()))
		return nil, array[0].Count, errors.DataBaseError
	}
	return ret, array[0].Count, nil
}

