// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"fmt"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"time"
)

type TStaff struct {
	Id int64 `gorm:"column:ID;PRIMARY_KEY" json:"id"`
	Code string `gorm:"column:Code" json:"code"`
	CorpCode string `gorm:"column:CorpCode" json:"corp_code"`
	CorpName string `gorm:"column:CorpName" json:"corp_name"`
	TeamSysNo int `gorm:"column:TeamSysNo" json:"team_sys_no"`
	TeamName string `gorm:"column:TeamName" json:"team_name"`
	WorkName string `gorm:"column:WorkName" json:"work_name"`
	IsTeamLeader string `gorm:"column:IsTeamLeader" json:"is_team_leader"`
	IDCardType int `gorm:"column:IDCardType" json:"id_card_type"`
	IDCardNumber string `gorm:"column:IDCardNumber" json:"id_card_number"`
	Age int `gorm:"column:Age" json:"age"`
	Gender string `gorm:"column:Gender" json:"gender"`
	Nation string `gorm:"column:Nation" json:"nation"`
	Address string `gorm:"column:Address" json:"address"`
	HeadImage string `gorm:"column:HeadImage" json:"head_image"`
	PoliticsType int `gorm:"column:PoliticsType" json:"politics_type"`
	CultureLevelType int `gorm:"column:CultureLevelType" json:"colture_level_type"`
	GrantOrg string `gorm:"column:GrantOrg" json:"grant_org"`
	WorkType string `gorm:"column:WorkType" json:"work_type"`
	NativePlace string `gorm:"column:NativePlace" json:"native_place"`
	Mobile string `gorm:"column:Mobile" json:"mobile"`
	IssueCardPicUrl string `gorm:"column:IssueCardPicUrl" json:"issue_card_pic_url"`
	HasContract string `gorm:"column:HasContract" json:"has_contract"`
	HasBuyInsurance string `gorm:"column:HasBuyInsurance" json:"has_buy_insurance"`
	Remark string `gorm:"column:Remark" json:"remark"`
	ProjectId int64 `gorm:"column:ProjectId" json:"project_id"`
	WorkNo string `gorm:"column:WorkNo" json:"work_no"`
	Sn string `gorm:"column:Sn" json:"sn"`
	CreatedAt time.Time `gorm:"column:CreatedAt" json:"created_at"`
	UpdatedAt time.Time `gorm:"column:UpdatedAt" json:"updated_at"`
}

type StaffListItem struct {
	TStaff
	StaffType int
}

func (TStaff) TableName() string {
	return "db_smart_attendance.ProjectWorkInfo"
}

func (p *TStaff) List(db *gorm.DB, filter string, projectId int64, page int32, pageSize int32) (list []StaffListItem, total int64, err error) {
	offset := (page - 1) *pageSize
	where := ""
	whereArray := []string{}
	type ResultCount struct {
		Count int64
	}
	sql := fmt.Sprintf("select t1.*,t2.PType as staff_type from db_smart_attendance.ProjectWorkInfo as t1 left join db_smart_attendance.ProjectPMInfo as t2 on t1.ProjectId=t2.ProjectId and t1.IDCardNumber=t2.PMIDCardNumber")
	countSql := fmt.Sprintf("select count(1) as count from db_smart_attendance.ProjectWorkInfo as t1 left join db_smart_attendance.ProjectPMInfo as t2 on t1.ProjectId=t2.ProjectId and t1.IDCardNumber=t2.PMIDCardNumber")
	if projectId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.ProjectId=%d", projectId))
	}
	if filter != "" {
		whereArray = append(whereArray, fmt.Sprintf("(t1.WorkName like '%%%s%%') ", filter))
	}
	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf(" where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}
	sql = fmt.Sprintf("%s %s limit %d offset %d", sql, where, pageSize, offset)
	countSql = fmt.Sprintf("%s %s", countSql, where)

	array := []ResultCount{}
	ret := []StaffListItem{}

	err = db.Raw(countSql).Scan(&array).Error

	if err != nil {
		logger.Error("mysql",
			zap.String("sql", countSql),
			zap.String("error", err.Error()))
		return  nil, 0, errors.DataBaseError
	}
	if len(array) == 0 {
		return nil, 0, nil
	}
	if array[0].Count == 0 {
		return nil, 0, nil
	}

	err = db.Raw(sql).Scan(&ret).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("error", err.Error()))
		return nil, array[0].Count, errors.DataBaseError
	}
	return ret, array[0].Count, nil
}
