package model

import (
	"fmt"
	"github.com/jinzhu/gorm"
	"time"
)

type TaskProcessingInfo struct {
	ID int64 `gorm:"column:ID;PRIMARY_KEY"`
	Code string `gorm:"column:Code"`
	TaskID string `gorm:"column:TaskID"`
	TaskName string `gorm:"column:TaskName"`
	AssignmentDate time.Time `gorm:"column:AssignmentDate"`
	FinishDate time.Time `gorm:"column:FinishDate"`
	ProcessingPerson string `gorm:"column:processingperson"`
	Reviewer string `gorm:"column:Reviewer"`
	ProcessingDate time.Time `gorm:"column:ProcessingDate"`
	ProcessingResults string `gorm:"column:ProcessingResults"`
	Status int32 `gorm:"column:Status"`
	ProcessingResultsImg string `gorm:"column:ProcessingResultsImg"`
	//AlarmID int64 `gorm:"column:AlarmID"`
	Feedback string `gorm:"column:Feedback"`
	ProjectID int64 `gorm:"column:ProjectID"`
	TaskDesc string `gorm:"column:TaskDesc"`
	TaskType int32 `gorm:"column:TaskType"`
	TaskImg string `gorm:"column:TaskImg"`
	Pipeline string `gorm:"column:Pipeline"`
	ApproveDate time.Time `gorm:"column:ApproveDate"`
	ApproveImg string `gorm:"column:ApproveImg"`
	Creater string `gorm:"column:Creater"`
}

const (
	PipelineCreate = 1
	PipelineProcessing = 2
	PipelineApprove = 3

)

type TaskPipeline struct {
	PipeType int32 `json:"pipe_type"`
	Person string `json:"person"`
	Desc string `json:"desc"`
	Img string `json:"img"`
	PipeTime string `json:"pipe_time"`
	Status string `json:"status"`
}

func (TaskProcessingInfo) TableName() string {
	return "TaskProcessingInfo"
}

func (p *TaskProcessingInfo) List(db *gorm.DB, where map[string]interface{}, or map[string]interface{}, page int32, pageSize int32) (list []TaskProcessingInfo, err error) {

	if len(where) > 0 || len(or) > 0 {
		cond, val, err := whereBuildAndOr(where, or)
		if err != nil {
			return list, err
		}
		if pageSize < 0 {
			result := db.Table(p.TableName()).Where(cond, val...).Order("AssignmentDate desc").Find(&list)

			return list, result.Error
		}
		offset := (page - 1) * pageSize
		result := db.Table(p.TableName()).Where(cond, val...).Order("AssignmentDate desc").Limit(pageSize).Offset(offset).Find(&list)

		return list, result.Error
	}
	if pageSize < 0 {
		result := db.Table(p.TableName()).Order("AssignmentDate desc").Find(&list)

		return list, result.Error
	}
	offset := (page - 1) * pageSize
	result := db.Table(p.TableName()).Order("AssignmentDate desc").Limit(pageSize).Offset(offset).Find(&list)
	return list, result.Error
}


func (p *TaskProcessingInfo) Insert(db *gorm.DB) error {
	return db.Table(p.TableName()).Create(p).Error
}

func (p *TaskProcessingInfo) Del(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
}

func (p *TaskProcessingInfo) Find(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).First(p).Error
}

func (p *TaskProcessingInfo) FindOr(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) error {
	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).First(p).Error
}

func (p *TaskProcessingInfo) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	return db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
}


func (p *TaskProcessingInfo) Count(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) (int64, error) {
	if len(where) > 0 || len(or) > 0{
		cond, val, err := whereBuildAndOr(where, or)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error

		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	return ret, err
}

type TaskListRequest struct {
	Filter string
	Status []int32
	Page int32
	PageSize int32
	Start int64
	End int64
	ProjectId int64
	TaskType int32
	Now time.Time
	Overdue int32
	Source int32
}

type TaskListItem struct {
	TaskProcessingInfo
	ProjectName string
	SafetyRecordNo string
}

func taskListSql(req *TaskListRequest)(string, string, []interface{}) {
	sql := fmt.Sprintf("select t1.*, t2.Name as project_name, t2.SafetyNo as safety_record_no from TaskProcessingInfo as t1 left join ProjectInfo as t2 on t1.ProjectID = t2.ID")
	countSql := fmt.Sprintf("select count(1) as count from TaskProcessingInfo as t1 left join ProjectInfo as t2 on t1.ProjectID = t2.ID")
	offset := int64(req.Page - 1) *int64(req.PageSize)
	whereArray := []string{}
	args := []interface{}{}
	if len(req.Status) > 0 {
		args = append(args, req.Status)
		whereArray = append(whereArray, fmt.Sprintf("t1.Status in(?)"))
	}
	if req.Filter != "" {
		whereArray = append(whereArray, fmt.Sprintf("t1.TaskName like '%%%s%%' or t1.TaskID like '%%%s%%'", req.Filter, req.Filter))
	}
	if req.Start > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.AssignmentDate >= '%s'", time.Unix(req.Start, 0).Format("2006-01-02 15:04:05")))
	}
	if req.End > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.AssignmentDate < '%s'", time.Unix(req.End, 0).Format("2006-01-02 15:04:05")))
	}
	if req.ProjectId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.ProjectID = %d", req.ProjectId))
	}
	if req.TaskType > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.TaskType=%d", req.TaskType))
	}
	if req.Overdue == 1 {
		whereArray = append(whereArray, fmt.Sprintf("((t1.ProcessingDate='0000-00-00 00:00:00' and t1.FinishDate<'%s') || t1.ProcessingDate > t1.FinishDate)", req.Now.Format("2006-01-02 15:04:05")))
	}
	if req.Overdue == 2 {
		whereArray = append(whereArray, fmt.Sprintf("((t1.ProcessingDate <> '0000-00-00 00:00:00'and t1.ProcessingDate <= t1.FinishDate)||(t1.ProcessingDate='0000-00-00 00:00:00' and t1.FinishDate>='%s'))", req.Now.Format("2006-01-02 15:04:05")))
	}
	where := ""
	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf("where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}

	sql = fmt.Sprintf("%s %s order by t1.AssignmentDate desc limit %d offset %d", sql, where, req.PageSize, offset)
	countSql = fmt.Sprintf("%s %s", countSql, where)
	return sql, countSql, args

}

func TaskList(db *gorm.DB, req *TaskListRequest) ([]TaskListItem, int64, error) {
	type ResultCount struct {
		Count int64
	}
	array := []ResultCount{}
	ret := []TaskListItem{}
	var err error

	sql, countSql, args := taskListSql(req)

	err = db.Raw(countSql, args...).Scan(&array).Error
	if err != nil {
		return nil, 0, err
	}
	if len(array) == 0 {
		return nil, 0, nil
	}
	if array[0].Count == 0 {
		return nil, 0, nil
	}

	err = db.Raw(sql, args...).Scan(&ret).Error
	if err != nil {
		return nil, array[0].Count, err
	}
	return ret, array[0].Count, nil
}
