// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"fmt"
	"github.com/jinzhu/gorm"
	"smart-site-management/consts"
	"time"
)

type Vssdevtbl struct {
	Id int64 `gorm:"column:ID;PRIMARY_KEY" json:"id" form:"id"`
	Devpubid string `gorm:"column:DevPubID" json:"devpubid" form:"devpubid"`
	Devnickname string `gorm:"column:DevNickname" json:"devnickname" form:"devnickname"`
	Username string `gorm:"column:UserName" json:"username" form:"username"`
	Passwd string `gorm:"column:Passwd" json:"passwd" form:"passwd"`
	Corpid string `gorm:"column:CorpID" json:"corpid" form:"corpid"`
	Ip string `gorm:"column:IP" json:"ip" form:"ip"`
	Port int64 `gorm:"column:Port" json:"port" form:"port"`
	Alive int64 `gorm:"column:Alive" json:"alive" form:"alive"`
	Subcatalogtime int64 `gorm:"column:SubCatalogTime" json:"subcatalogtime" form:"subcatalogtime"`
	Devmode string `gorm:"column:DevMode" json:"devmode" form:"devmode"`
	Mediatransport string `gorm:"column:MediaTransport" json:"mediatransport" form:"mediatransport"`
	Dmarker int64 `gorm:"column:DMarker" json:"dmarker" form:"dmarker"`
	UpdateTime time.Time `gorm:"column:updatetime" json:"updatetime" form:"updatetime"`
	ChannelCount int `gorm:"column:ChannelCount"`
	ProviderId int64 `gorm:"column:ProviderId"`
	ProjectId int64 `gorm:"column:ProjectId"`
	VerifyStatus int `gorm:"column:VerifyStatus"`
	VerifyTime time.Time `gorm:"column:VerifyTime"`
	CreatedAt time.Time `gorm:"column:CreatedAt"`
}

func (Vssdevtbl) TableName() string {
	return "VSSDevTbl"
}

func (p *Vssdevtbl) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	return db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
}

type VedioDeviceItem struct {
	Id                   int64
	Sn                   string
	TypeCode             int32
	TypeName             string
	ProjectName          string
	ProjectId            int64
	SafetyRecordNo       string
	CreatedTime          time.Time
	Status               int32
	State int32
	ProjectApproveTime time.Time
	Key string
	ProviderName string
	SocialCode string
	Name string
	Ip string
	Port int
	MediaTransport string
	ChannelCount int

}


func vedioDeviceListSql(req VedioDeviceListRequest)(string, string, []interface{}) {
	sql := fmt.Sprintf("select t1.ID as id, t1.DevNickName as name, t1.DevPubID as sn, t1.ProjectId as project_id, t1.VerifyTime as project_approve_time, t1.MediaTransport as media_transport, t1.IP as ip, t1.Port as port, t1.ChannelCount as channel_count, "+
		"t1.CreatedAt as created_time, t1.Alive as state, t1.VerifyStatus as status, t2.SafetyNo as safety_record_no, t2.Name as project_name, t3.Name as provider_name, t3.SocialCode as social_code from VSSDevTbl as t1 left join ProjectInfo as t2 on t1.ProjectId=t2.ID left join Provider as t3 on t1.ProviderId=t3.ID")
	countSql := fmt.Sprintf("select count(1) as count from VSSDevTbl as t1 left join ProjectInfo as t2 on t1.ProjectId=t2.ID")
	args := []interface{}{}
	whereArray := []string{}
	if req.ProviderId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.ProviderId=%d", req.ProviderId))
	}

	if req.Filter != "" {
		whereArray = append(whereArray, fmt.Sprintf("(t1.DevPubID like '%%%s%%' or t1.DevNickName like '%%%s%%' )",  req.Filter, req.Filter))
	}
	if req.Cid > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t2.Cid=%d", req.Cid))
	}

	if req.ProjectId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.ProjectId = %d", req.ProjectId))
	}

	if req.State >= 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.Alive=%d", req.State))
	}
	if req.CanDel {
		req.IsAll = true
		req.StatusFilter = []int32{consts.DeviceStatusAddAuditted}
		whereArray = append(whereArray, fmt.Sprintf("(select count(1) from DeviceDelJob where DeviceId=t1.ID and (Status = 0 or Status = 1)) = 0"))
	}
	if len(req.StatusFilter) > 0 {
		args = append(args, req.StatusFilter)
		whereArray = append(whereArray, fmt.Sprintf("t1.VerifyStatus in(?)"))
	}
	where := ""

	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf(" where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}
	offset := (req.Page - 1) *int32(PageSize)
	if req.IsAll {
		sql = fmt.Sprintf("%s %s order by t1.CreatedAt desc", sql, where)
		countSql = fmt.Sprintf("%s %s", countSql, where)
	} else {
		sql = fmt.Sprintf("%s %s order by t1.CreatedAt desc limit %d offset %d", sql, where, PageSize, offset)
		countSql = fmt.Sprintf("%s %s", countSql, where)
	}
	return sql, countSql, args
}


type VedioDeviceListRequest struct {
	ProviderId int64
	Filter string
	StatusFilter []int32
	IsAll bool
	CanDel bool
	Page int32
	ProjectId int64
	Cid int64
	State int32
}

func  VedioDeviceList(db *gorm.DB, req VedioDeviceListRequest) ([]VedioDeviceItem, int64, error) {
	type ResultCount struct {
		Count int64
	}
	array := []ResultCount{}
	ret := []VedioDeviceItem{}
	var err error

	sql, countSql, args := vedioDeviceListSql(req)

	err = db.Raw(countSql, args...).Scan(&array).Error
	if err != nil {
		return nil, 0, err
	}
	if len(array) == 0 {
		return nil, 0, nil
	}
	if array[0].Count == 0 {
		return nil, 0, nil
	}

	err = db.Raw(sql, args...).Scan(&ret).Error
	if err != nil {
		return nil, array[0].Count, err
	}
	return ret, array[0].Count, nil
}
