// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"fmt"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"time"
	"github.com/jaryhe/gopkgs/logger"
	"encoding/json"
	"smart-site-management/consts"
)

type Vsschanneltbl struct {
	ID int64 `gorm:"column:ID"`
	Devpubid string `gorm:"column:DevPubID"`
	Restype int64 `gorm:"column:ResType"`
	Nickname string `gorm:"column:Nickname"`
	Chanpubid string `gorm:"column:ChanPubID"`
	Alive int64 `gorm:"column:Alive"`
	Corpid string `gorm:"column:CorpID"`
	Model string `gorm:"column:Model"`
	Owner string `gorm:"column:Owner"`
	Civilcode string `gorm:"column:CivilCode"`
	Address string `gorm:"column:Address"`
	Parental int64 `gorm:"column:Parental"`
	Parentid string `gorm:"column:ParentId"`
	Ip string `gorm:"column:IP"`
	Port int64 `gorm:"column:Port"`
	Longitude float64 `gorm:"column:Longitude"`
	Latitude float64 `gorm:"column:Latitude"`
	Altitude float64 `gorm:"column:Altitude"`
	Ptztype int64 `gorm:"column:PTZType"`
	Roomtype int64 `gorm:"column:RoomType"`
	Directiontype int64 `gorm:"column:DirectionType"`
	Streamtype int64 `gorm:"column:StreamType"`
	Chanrtspurl string `gorm:"column:ChanRtspUrl"`
	Realrtspurl string `gorm:"column:RealRtspUrl"`
	Dmarker int64 `gorm:"column:DMarker"`
	Updatetime time.Time `gorm:"column:UpdateTime"`
}

func (Vsschanneltbl) TableName() string {
	return "VSSChannelTbl"
}

func (p *Vsschanneltbl) Count(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) (int64, error) {
	if len(where) > 0 || len(or) > 0 {
		cond, val, err := whereBuildAndOr(where, or)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select count(1) from VSSChannelTbl"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			return  0, errors.DataBaseError
		}

		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select count(1) from VSSChannelTbl"),
			zap.String("error", err.Error()))
		return  0, errors.DataBaseError
	}
	return ret, err
}

func (p *Vsschanneltbl) Find(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err =  db.Table(p.TableName()).Where(cond, val...).First(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select from t_VSSChannelTbl "),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return nil
		}
		return  errors.DataBaseError
	}
	return err
}

func (p *Vsschanneltbl) List(db *gorm.DB, where map[string]interface{}, or map[string]interface{}, page int32, pageSize int32) (list []Vsschanneltbl, err error) {
	offset := (page - 1) *pageSize
	if len(where) > 0 || len(or) > 0 {
		cond, val, err := whereBuildAndOr(where, or)
		if err != nil {
			return list, err
		}

		err = db.Table(p.TableName()).Where(cond, val...).Limit(pageSize).Offset(offset).Find(&list).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select * from VSSChannelTbl"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			if err == gorm.ErrRecordNotFound {
				return list, nil
			}
			return  list, errors.DataBaseError
		}

		return list, err
	}
	err = db.Table(p.TableName()).Limit(pageSize).Offset(offset).Find(&list).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select * from VSSChannelTbl"),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return list, nil
		}
		return  list, errors.DataBaseError
	}

	return list, err

}


func (p *Vsschanneltbl) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	return db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
}

func GetProjectIdByChannel(db *gorm.DB, id int64) (int64, error) {
	sql := fmt.Sprintf("select t2.ProjectId from VSSChannelTbl as t1 left join DeviceAll as t2 on t1.DevPubID = t2.SN and t2.DeviceCode=%d where t1.ID=?", DeviceTypeVedio)
	type Result struct {
		ProjectId int64
	}
	array := []Result{}
	err := db.Raw(sql, id).Scan(&array).Error
	if err != nil {
		return 0, err
	}
	if len(array) == 0 {
		return 0, nil
	}
	return array[0].ProjectId, nil
}

func ChannelListByProject(db *gorm.DB, projectId int64, page int32, pageSize int32, filter string, isUnbind bool)(list []Vsschanneltbl, total int64, err error) {
	sql := fmt.Sprintf("select t1.* from VSSChannelTbl as t1 left join DeviceAll as t2 on t2.SN = t1.DevPubID where t2.ProjectId=%d and t2.DeviceCode=%d and t2.VerifyStatus = %d limit %d offset %d", projectId, DeviceTypeVedio, consts.DeviceStatusAddAuditted, pageSize, (page - 1)*pageSize)
	countSql := fmt.Sprintf("select count(1) as count from VSSChannelTbl as t1 left join DeviceAll as t2 on t2.SN = t1.DevPubID where t2.ProjectId=%d and t2.DeviceCode=%d and t2.VerifyStatus=%d", projectId, DeviceTypeVedio, consts.DeviceStatusAddAuditted)
	if filter != "" {
		sql = fmt.Sprintf("select t1.* from VSSChannelTbl as t1 left join DeviceAll as t2 on t2.SN = t1.DevPubID where t2.ProjectId=%d and t2.DeviceCode=%d and t2.VerifyStatus = %d and (LOWER(t1.Nickname) like '%%%s%%' or t1.ChanPubID like '%%%s%%') limit %d offset %d", projectId, DeviceTypeVedio, consts.DeviceStatusAddAuditted, filter, filter, pageSize, (page - 1)*pageSize)
		countSql = fmt.Sprintf("select count(1) as count from VSSChannelTbl as t1 left join DeviceAll as t2 on t2.SN = t1.DevPubID where t2.ProjectId=%d and t2.DeviceCode=%d and t2.VerifyStatus = %d and (LOWER(t1.Nickname) like '%%%s%%' or t1.ChanPubID like '%%%s%%')", projectId, DeviceTypeVedio, consts.DeviceStatusAddAuditted,filter, filter)
	}
	if isUnbind {
		sql = fmt.Sprintf("select t1.* from VSSChannelTbl as t1 left join DeviceAll as t2 on t2.SN = t1.DevPubID where t2.ProjectId=%d and t2.DeviceCode=%d and t2.VerifyStatus = %d and t1.ID not in (select ChannelId from DeviceAll where DeviceCode<>%d and ChannelId > 0 )", projectId, DeviceTypeVedio, consts.DeviceStatusAddAuditted, DeviceTypeVedio)
		countSql = fmt.Sprintf("select count(1) as count from VSSChannelTbl as t1 left join DeviceAll as t2 on t2.SN = t1.DevPubID where t2.ProjectId=%d and t2.DeviceCode=%d and t2.VerifyStatus = %d and t1.ID not in (select ChannelId from DeviceAll where DeviceCode<>%d and ChannelId > 0 )", projectId, DeviceTypeVedio, consts.DeviceStatusAddAuditted, DeviceTypeVedio)

	}

	type ResultCount struct {
		Count int64
	}
	array := []ResultCount{}
	err = db.Raw(countSql).Scan(&array).Error

	if err != nil {
		logger.Error("mysql",
			zap.String("sql", countSql),
			zap.String("error", err.Error()))
		return  nil, 0, errors.DataBaseError
	}
	if len(array) == 0 {
		return nil, 0, nil
	}
	if array[0].Count == 0 {
		return nil, 0, nil
	}

	err = db.Raw(sql).Scan(&list).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("error", err.Error()))
		return nil, array[0].Count, errors.DataBaseError
	}
	return list, array[0].Count, nil
}
